@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -90,7 +90,7 @@ int bindport(const char *hostname, int port) {
 			 */
 			host_ip = hostent_get_ip(&hostcache, hostname);
 			if (host_ip == (unsigned long int) UINT_MAX) {
-				log(1, "Could not resolve %s: %s",
+				jlog(1, "Could not resolve %s: %s",
 					hostname, strerror(errno));
 				perror("Could not resolve the hostname");
 				return -1;
@@ -112,26 +112,26 @@ int bindport(const char *hostname, int port) {
 
 	shandle = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (shandle < 0) {
-		log(1, "Error creating socket to bind: %s", strerror(errno));
+		jlog(1, "Error creating socket to bind: %s", strerror(errno));
 		perror("Error creating socket to bind to");
 		return -1;
 	}
 	if (setsockopt(shandle, SOL_SOCKET, SO_REUSEADDR,
 				(void*) &one, sizeof(one)) < 0) {
-		log(3, "Error setting socket to SO_REUSEADDR");
+		jlog(3, "Error setting socket to SO_REUSEADDR");
 	}
 	if (bind(shandle, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-		log(1, "Error binding: %s", strerror(errno));
+		jlog(1, "Error binding: %s", strerror(errno));
 		perror("Error binding");
 		return -1;
 	}
 	if (listen(shandle, 5) < 0) {
-		log(1, "Error listening on the socket: %s", strerror(errno));
+		jlog(1, "Error listening on the socket: %s", strerror(errno));
 		perror("Error listening on a bound socket");
 		return -1;
 	}
 
-	log(6, "Listening on %s, port %d, fd %d", hostname, port, shandle);
+	jlog(6, "Listening on %s, port %d, fd %d", hostname, port, shandle);
 
 	if (changeid(UNPRIV, EUID,
 				"Changing id back (socket(), bind())") < 0) {
@@ -174,6 +174,19 @@ int waitclient(const char* hostnames, struct clientinfo* clntinfo) {
 		daemonize();
 	}
 
+	if (changeid(UNPRIV, EUID,
+				"Changing id back (socket(), bind())") < 0) {
+		return -1;
+	}
+
+	d_set = listen_on_ifaces(hostnames, clntinfo);
+	if (d_set.maxfd < 0) {
+		jlog(8, "d_set.maxfd was negative: %d", d_set.maxfd);
+		return -1;
+	}
+
+	/* we have successfully bound */
+
 	/* become root again - use our function instead of plain
 	 * setuid() for the logging */
 	if (changeid(PRIV, UID, "Changing ID to root (pidfile)") < 0) {
@@ -192,14 +205,8 @@ int waitclient(const char* hostnames, struct clientinfo* clntinfo) {
 			 * pidfile */
 			atexit(removepidfile);
 		} else {
-			log(2, "Error creating pidfile %s", option);
+			jlog(2, "Error creating pidfile %s", option);
 		}
-		umask(0);
-	}
-
-	if (changeid(UNPRIV, EUID,
-				"Changing id back (socket(), bind())") < 0) {
-		return -1;
 	}
 
 	/* this has to be done for the daemonization. We do it now after
@@ -212,14 +219,6 @@ int waitclient(const char* hostnames, struct clientinfo* clntinfo) {
 		return -1;
 	}
 
-	d_set = listen_on_ifaces(hostnames, clntinfo);
-	if (d_set.maxfd < 0) {
-		log(8, "d_set.maxfd was negative: %d", d_set.maxfd);
-		return -1;
-	}
-
-	/* we have successfully bound */
-
 	sa.sa_handler = childterm;
 	chlds_exited = 0;
 	sigemptyset (&sa.sa_mask);
@@ -244,7 +243,7 @@ int waitclient(const char* hostnames, struct clientinfo* clntinfo) {
 			 * endless loop. The danger of the programm falling
 			 * into an infinite loop consuming all cpu time is
 			 * too big... */
-			log(8, "get_connecting_socket() returned error code");
+			jlog(8, "get_connecting_socket() returned error code");
 			return -1;
 		}
 
@@ -266,7 +265,7 @@ int waitclient(const char* hostnames, struct clientinfo* clntinfo) {
 		}
 		if (srvinfo.multithread) {
 			if ((chldpid = fork()) < 0) {
-				log(1, "Error forking: %s", strerror(errno));
+				jlog(1, "Error forking: %s", strerror(errno));
 				close(ahandle);
 				return -1;
 			}
@@ -281,7 +280,7 @@ int waitclient(const char* hostnames, struct clientinfo* clntinfo) {
 			}
 			if (chldpid == 0) {
 				/* child process */
-				log(8, "forked to pid %d", getpid());
+				jlog(8, "forked to pid %d", getpid());
 			}
 		}
 		if (!srvinfo.multithread || chldpid == 0) {
@@ -322,12 +321,12 @@ int get_connecting_socket(struct descriptor_set d_set) {
 					get_chld_pid();
 				}
 				if (should_read_config) {
-					log(9, "Re-reading configuration");
+					jlog(9, "Re-reading configuration");
 					reread_config();
 				}
 				continue;
 			}
-			log(1, "select() failed: %s, nfd: %d", strerror(errno), nfd);
+			jlog(1, "select() failed: %s, nfd: %d", strerror(errno), nfd);
 			return -1;
 		}
 	}
@@ -342,10 +341,24 @@ int get_connecting_socket(struct descriptor_set d_set) {
 	while(1) {
 		ahandle = accept(shandle, (struct sockaddr *) &sin, &size);
 		if (ahandle < 0) {
-			if (errno == EINTR) {
-				continue;
+			switch(errno) {
+				case EINTR:        /* signal */
+				case ECONNRESET:   /* client quit before three-way-handshake */
+				case ENETDOWN:     /* the others: see manpage */
+#ifdef EPROTO
+				case EPROTO:
+#endif
+				case ENOPROTOOPT:
+				case EHOSTDOWN:
+#ifdef ENONET
+				case ENONET:
+#endif
+				case EHOSTUNREACH:
+				case EOPNOTSUPP:
+				case ENETUNREACH:
+					continue;
 			}
-			log(1, "accept() failed: %s", strerror(errno));
+			jlog(1, "accept() failed: %s", strerror(errno));
 			return -1;
 		}
 		break;
@@ -405,9 +418,9 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 	clntinfo->serversocket = -1;
 
 	clntinfo->clientsocket = sock_fd;
-	log(7, "Connection from %s", get_char_peer_ip(sock_fd));
+	jlog(7, "Connection from %s", get_char_peer_ip(sock_fd));
 	c_in = socketinfo_get_local_sin(sock_fd);
-	log(7, "Client tried to connect to %s on port %d",
+	jlog(7, "Client tried to connect to %s on port %d",
 					inet_ntoa(c_in.sin_addr),
 					ntohs(c_in.sin_port));
 
@@ -417,11 +430,11 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 
 	/* see if we are in transparent mode */
 	t_in = socketinfo_get_transparent_target_sin(sock_fd);
-	log(7, "Transparent target seems to be %s on port %d",
+	jlog(7, "Transparent target seems to be %s on port %d",
 					inet_ntoa(t_in.sin_addr),
 					ntohs(t_in.sin_port));
 
-	log(9, "Checking TAG_GLOBAL | TAG_FROM | TAG_PROXYIP | TAG_PROXYPORT | TAG_TIME | TAG_SERVERTYPE");
+	jlog(9, "Checking TAG_GLOBAL | TAG_FROM | TAG_PROXYIP | TAG_PROXYPORT | TAG_TIME | TAG_SERVERTYPE");
 	ret = config_shrink_config(peer_ip,	/* source IP */
 				-1,		/* dest IP */
 				(char*) 0,	/* dest name */
@@ -439,7 +452,7 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 				TAG_CONNECTED
 				);
 	if (ret != 0) {
-		log(2, "Error shrinking config data");
+		jlog(2, "Error shrinking config data");
 		return ret;
 	}
 
@@ -466,13 +479,13 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 			     NULL);
 
 		if (libwrap_allow || hosts_access(&req)) {
-			log(5, "%s is allowed by libwrap to connect.", 
+			jlog(5, "%s is allowed by libwrap to connect.", 
 					conv_ip_to_char(peer_ip));
 		}
 		else {
 			say(sock_fd, "500 You are not allowed by libwrap to "
 					"connect. Goodbye.\r\n");
-			log(5, "%s was not allowed to connect.",
+			jlog(5, "%s was not allowed to connect.",
 					conv_ip_to_char(peer_ip));
 			close(sock_fd);
 			return -2;
@@ -485,25 +498,25 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 
 			say(sock_fd, "500 You are not allowed to "
 				"connect. Goodbye.\r\n");
-			log(5, "%s was not allowed to connect.",
+			jlog(5, "%s was not allowed to connect.",
 						conv_ip_to_char(peer_ip));
 			close(sock_fd);
 			return -2;
 		} else {
-			log(6, "%s is allowed to connect.",
+			jlog(6, "%s is allowed to connect.",
 					conv_ip_to_char(peer_ip));
 		}
 	}
 
 	if (config_get_bool("transparent-proxy") && 
 			get_uint_peer_ip(sock_fd) == t_in.sin_addr.s_addr) {
-		log(4, "proxy loop detected - machine connects to itself, disabling transparent proxy support");
+		jlog(4, "proxy loop detected - machine connects to itself, disabling transparent proxy support");
 	}
 
-	log(9, "Proxy loop check: peer_ip: %s, t_in_ip: %s",
+	jlog(9, "Proxy loop check: peer_ip: %s, t_in_ip: %s",
 		get_char_peer_ip(sock_fd), inet_ntoa(t_in.sin_addr));
 #ifdef HAVE_LINUX_NETFILTER_IPV4_H
-	log(9, "HAVE_LINUX_NETFILTER_IPV4_H true, c_in_ip: %s, t_in_port: %d, "
+	jlog(9, "HAVE_LINUX_NETFILTER_IPV4_H true, c_in_ip: %s, t_in_port: %d, "
 		"c_in_port: %d", inet_ntoa(c_in.sin_addr),
 		ntohs(t_in.sin_port), ntohs(c_in.sin_port));
 #endif
@@ -520,11 +533,11 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 #endif
 		)) {
 
-		log(9, "Enabling transparent proxy");
+		jlog(9, "Enabling transparent proxy");
 		clntinfo->transparent = TRANSPARENT_YES;
 		clntinfo->transparent_destination = t_in;
 	} else {
-		log(9, "No transparent proxy support");
+		jlog(9, "No transparent proxy support");
 		clntinfo->transparent = TRANSPARENT_NO;
 		clntinfo->transparent_destination.sin_port = 0;
 		clntinfo->transparent_destination.sin_addr.s_addr = 0;
@@ -545,7 +558,7 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 
 			clntinfo->destination =
 			      strdup(config_get_option("transparent-forward"));
-			log(9, "Enabling transparent forward to %s",
+			jlog(9, "Enabling transparent forward to %s",
 					clntinfo->destination);
 		} else {
 			clntinfo->destination =
@@ -554,7 +567,7 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 		colon = strchr(clntinfo->destination, ':');
 		if (!colon) {
 			/* abort, shouldnt happen */
-			log(9, "Could not get transparent target properly");
+			jlog(9, "Could not get transparent target properly");
 			clntinfo->transparent = TRANSPARENT_NO;
 			return say_welcome(sock_fd);
 		}
@@ -567,7 +580,7 @@ int child_setup(int sock_fd, struct clientinfo *clntinfo) {
 				config_get_ioption("serverport", DEFAULTSERVERPORT);
 			/* reverse for logging */
 			colon--; *colon = ':';
-			log(6, "Could not parse dest/port to connect to: %s",
+			jlog(6, "Could not parse dest/port to connect to: %s",
 					clntinfo->destination);
 			return -1;
 		} else {
@@ -603,7 +616,7 @@ int say_welcome(int sock_fd) {
 		welcmsg = prependcode(welcome, 220);
 		enough_mem(welcmsg);
 
-		log(9, "Saying this text as welcomeline: %s", welcmsg);
+		jlog(9, "Saying this text as welcomeline: %s", welcmsg);
 		say(sock_fd, welcmsg);
 		free(welcmsg);
 	} else {
@@ -647,7 +660,7 @@ struct descriptor_set listen_on_ifaces(const char* hostnames,
 	while ((part = quotstrtok(hostnames2, WHITESPACES, &offset))) {
 		portdel = strchr(part, ':');
 		if (!portdel) {
-			log(3, "Invalid IP/Port specification: %s", part);
+			jlog(3, "Invalid IP/Port specification: %s", part);
 			free(part);
 			continue;
 		}
@@ -655,19 +668,19 @@ struct descriptor_set listen_on_ifaces(const char* hostnames,
 		errno = 0;
 		port = strtol(portdel+1, (char**) 0, 10);
 		if (errno || port > 65535 || port <= 0) {
-			log(4, "Invalid port specification: %s. "
+			jlog(4, "Invalid port specification: %s. "
 			   "Using default value %d", portdel, DEFAULTBINDPORT);
 			port = DEFAULTBINDPORT;
 		}
 		portdel = (char*) 0;
 
-		log(9, "binding to %s, port %d", part, port);
+		jlog(9, "binding to %s, port %d", part, port);
 
 		shandle = bindport(part, port);
 		free(part);
 		part = (char*) 0;
 		if (shandle < 0) {
-			log(8, "Could not bind: %s", strerror(errno));
+			jlog(8, "Could not bind: %s", strerror(errno));
 			free(hostnames2);
 			FD_ZERO(&d_set.set);
 			d_set.maxfd = -1;
@@ -681,7 +694,7 @@ struct descriptor_set listen_on_ifaces(const char* hostnames,
 	hostnames2 = (char*) 0;
 
 	if (clntinfo->boundsocket_niface == 0) {
-		log(2, "No interfaces found to bind to");
+		jlog(2, "No interfaces found to bind to");
 		FD_ZERO(&d_set.set);
 		d_set.maxfd = -1;
 		return d_set;
