@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -45,10 +45,10 @@ int handle_cmds(struct clientinfo *clntinfo) {
 	conn_info.lcs = &lcs;
 	conn_info.clntinfo = clntinfo;
 	clntinfo->cachefd = -1;
-	log(9, "setting dataclientsock to -1 (initial)");
+	jlog(9, "setting dataclientsock to -1 (initial)");
 	clntinfo->dataclientsock = clntinfo->dataserversock = -1;
 	clntinfo->dataport = socketinfo_get_local_port(clntinfo->clientsocket) - 1;
-	log(9, "dataport is set to %d", clntinfo->dataport);
+	jlog(9, "dataport is set to %d", clntinfo->dataport);
 	clntinfo->portcmd = (char*) 0;
 	ss = clntinfo->serversocket;
 	cs = clntinfo->clientsocket;
@@ -81,7 +81,7 @@ contin:
 		ss = clntinfo->serversocket;
 
 		if (timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+			jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 			return -1;
 		}
 		if (buffer) {
@@ -98,7 +98,7 @@ contin:
 		buffer = readline(cs);
 
 		if (!buffer && timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+			jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 			err_time_readline(cs);
 			return -1;
 		}
@@ -128,10 +128,16 @@ contin:
 		if (passcmd_check(lcs.method) == 0) {
 			say(clntinfo->clientsocket,
 				"500 Command not implemented\r\n");
-			log(8, "rejected command %s", lcs.method);
+			jlog(8, "rejected command %s", lcs.method);
 			goto contin;
 		}
 
+		/* We must not perform ASCII<->binary conversions if we are
+		 * transferring a server list like LIST or NLST. Assume we
+		 * don't have one and set it to 1 in the handler for the
+		 * respective commands. */
+		clntinfo->serverlisting = 0;
+
 		/* Retrieve a file */
 		clntinfo->mode = RETR;
 		i = 0;
@@ -140,7 +146,7 @@ contin:
 			if (checkbegin(buffer, cmdhandler[i].cmd)) {
 				int ret = (cmdhandler[i].func)
 						(buffer, &conn_info);
-				if (checkbegin(buffer, "PASS ")) {
+				if (checkbegin(buffer, "PASS")) {
 					memset(buffer, 0, strlen(buffer));
 				}
 				switch (ret) {
@@ -207,7 +213,7 @@ char* getftpwd(struct clientinfo *clntinfo) {
 	/*  257 "/home/joe/hi""ho lo di'da" is current directory. */
 
 	if ( ! checkdigits(answer, 257)) {
-		log(4, "PWD failed: %s", answer);
+		jlog(4, "PWD failed: %s", answer);
 		free(answer);
 		return (char*) 0;
 	}
@@ -215,7 +221,7 @@ char* getftpwd(struct clientinfo *clntinfo) {
 	dirend = strrchr(answer, '"');
 
 	if ( ! dirstart  ||  ! dirend ) {
-		log(4, "Could not parse PWD command: %s", answer);
+		jlog(4, "Could not parse PWD command: %s", answer);
 		free(answer);
 		return (char*) 0;
 	}
@@ -255,7 +261,7 @@ time_t getftpmdtm(const char* filename, struct clientinfo *clntinfo) {
 	free(cmd);
 	answer = ftp_readline(clntinfo->serversocket);
 	if ( ! checkdigits(answer, 213)) {
-		log(4, "Error reading MDTM answer: %s", answer);
+		jlog(4, "Error reading MDTM answer: %s", answer);
 		free(answer);
 		return (time_t)(-1);
 	}
@@ -266,7 +272,7 @@ time_t getftpmdtm(const char* filename, struct clientinfo *clntinfo) {
 			&tms.tm_hour, &tms.tm_min, &tms.tm_sec);
 
 	if (i != 6) {
-		log(4, "Error parsing MDTM answer: %s", answer);
+		jlog(4, "Error parsing MDTM answer: %s", answer);
 		free(answer);
 		return (time_t)(-1);
 	}
@@ -296,14 +302,14 @@ unsigned long int getftpsize(char* filename, struct clientinfo *clntinfo) {
 	free(cmd);
 	answer = ftp_readline(clntinfo->serversocket);
 	if ( ! checkdigits(answer, 213)) {
-		log(4, "Error reading SIZE answer: %s", answer);
+		jlog(4, "Error reading SIZE answer: %s", answer);
 		free(answer);
 		return 0;
 	}
 
-	i = sscanf(answer, "213 %ld", &size);
+	i = sscanf(answer, "213 %lu", &size);
 	if (i != 1) {
-		log(4, "Error parsing SIZE answer: %s", answer);
+		jlog(4, "Error parsing SIZE answer: %s", answer);
 		free(answer);
 		return 0;
 	}
@@ -322,18 +328,18 @@ int passcmd(const char* buffer, struct clientinfo *clntinfo) {
 	sendbufsize = strlen(buffer) + 3;
 	sendbuf = (char*) malloc(sendbufsize);
 	snprintf(sendbuf, sendbufsize, "%s\r\n", buffer);
-	log(9, "Send (server - %d): %s", ss, sendbuf);
+	jlog(9, "Send (server - %d): %s", ss, sendbuf);
 	say(ss, sendbuf);
 	free(sendbuf);
 	lcs.complete = 0;
 	last = passall(ss, cs);
 	if (last) {
 		lcs.respcode = getcode(last);
-		log(9, "Send (client - %d): %s", cs, last);
+		jlog(9, "Send (client - %d): %s", cs, last);
 	}
 	if (!last) {
 		if (timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+			jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 			err_time_readline(cs);
 		} else {
 			err_readline(cs);
@@ -390,7 +396,7 @@ int transfer_negotiate(struct clientinfo *clntinfo) {
 		free(clntinfo->portcmd);
 		clntinfo->portcmd = (char*) 0;
 		if (ret) {
-			log(2, "Error in activeclient()");
+			jlog(2, "Error in activeclient()");
 		}
 	}
 
@@ -422,11 +428,11 @@ int transfer_negotiate(struct clientinfo *clntinfo) {
 					 clntinfo->dataclientsock) + 1,
 					&acc_set, NULL, NULL, &tmo);
 			if (ret < 0) {
-				log(2, "Select() error1: %s", strerror(errno));
+				jlog(2, "Select() error1: %s", strerror(errno));
 				return -1;
 			}
 			if (ret == 0) {
-				log(2, "Connection timed out");
+				jlog(2, "Connection timed out");
 				lcs.respcode = 500;
 				say(cs, "500 Connection timed out\r\n");
 				return -1;
@@ -437,7 +443,7 @@ int transfer_negotiate(struct clientinfo *clntinfo) {
 				ret = accept(clntinfo->dataserversock,
 					(struct sockaddr*) &sin, &count);
 				if (ret < 0) {
-					log(2, "Error in accept() #1: %s",
+					jlog(2, "Error in accept() #1: %s",
 							strerror(errno));
 					return -1;
 				}
@@ -450,7 +456,7 @@ int transfer_negotiate(struct clientinfo *clntinfo) {
 				ret = accept(clntinfo->dataclientsock,
 					(struct sockaddr*) &sin, &count);
 				if (ret < 0) {
-					log(2, "Error in accept() #2: %s",
+					jlog(2, "Error in accept() #2: %s",
 							strerror(errno));
 					return -1;
 				}
@@ -478,11 +484,11 @@ int transfer_negotiate(struct clientinfo *clntinfo) {
 		ret = select(*clntinfo->waitforconnect + 1, &acc_set,
 				NULL, NULL, &tmo);
 		if (ret < 0) {
-			log(2, "Select() error2: %s", strerror(errno));
+			jlog(2, "Select() error2: %s", strerror(errno));
 			return -1;
 		}
 		if (ret == 0) {
-			log(2, "Connection timed out");
+			jlog(2, "Connection timed out");
 			lcs.respcode = 500;
 			say(cs, "500 Connection timed out\r\n");
 			return -1;
@@ -491,7 +497,7 @@ int transfer_negotiate(struct clientinfo *clntinfo) {
 		ret = accept(*clntinfo->waitforconnect,
 				(struct sockaddr*) &sin, &count);
 		if (ret < 0) {
-			log(2, "Error in accept() #3: %s",
+			jlog(2, "Error in accept() #3: %s",
 					strerror(errno));
 			return -1;
 		}
@@ -523,6 +529,7 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 	int n, ret, error = 0, aborted = 0;
 	int maxfd;
 	int fdflags;
+	int strictasciiconversion = 1;
 	fd_set readset, writeset, exceptset;
 	struct timeval readtime;
 	struct timeval writetime;
@@ -533,6 +540,7 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 	readtime.tv_usec = 0;
 	writetime.tv_sec = config_get_ioption("transfertimeout", 300);
 	writetime.tv_usec = 0;
+	strictasciiconversion = config_get_bool("strictasciiconversion");
 
 	enough_mem(buffer);
 	FD_ZERO(&readset);
@@ -551,16 +559,16 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 	FD_SET(cs, &readset);
 	FD_SET(clntinfo->dataclientsock, &writeset);
 
-	log(7, "Throughputrate is %3.3f", clntinfo->throughput);
+	jlog(7, "Throughputrate is %3.3f", clntinfo->throughput);
 
 	/* set the descriptor nonblocking */
 	if ((fdflags = fcntl(clntinfo->dataclientsock, F_GETFL)) < 0) {
-		log(2, "Error getting fcntl() flags");
+		jlog(2, "Error getting fcntl() flags");
 		fdflags = 0;
 	}
 	ret = fcntl(clntinfo->dataclientsock, F_SETFL, fdflags | O_NONBLOCK);
 	if (ret < 0) {
-		log(2, "Error setting fd to nonblocking");
+		jlog(2, "Error setting fd to nonblocking");
 	}
 
 	totwritten = 0;
@@ -571,7 +579,7 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 	sigaddset(&sigset, SIGCHLD);
 	ret = sigprocmask(SIG_BLOCK, &sigset, &oldset);
 	if (ret < 0) {
-		log(3, "sigprocmask() error: %s", strerror(errno));
+		jlog(3, "sigprocmask() error: %s", strerror(errno));
 	}
 
 	while(1) {
@@ -596,12 +604,15 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 		n = errno;
 		ret = sigprocmask(SIG_UNBLOCK, &sigset, &oldset);
 		if (ret < 0) {
-			log(3, "sigprocmask() error releasing the blocked"
+			jlog(3, "sigprocmask() error releasing the blocked"
 				" signals: %s", strerror(errno));
 		}
 		errno = n;
 
-		if (sret <= 0) {
+		if (sret < 0) {
+			break;
+		}
+		if (sret == 0 && !clntinfo->fromcache) {
 			break;
 		}
 
@@ -612,12 +623,12 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 			count = read(clntinfo->dataserversock,
 				buffer, TRANSMITBUFSIZE);
 			if (count == 0) {
-				log(8, "Read 0 bytes at %s (%d)", __FILE__, __LINE__);
+				jlog(8, "Read 0 bytes at %s (%d)", __FILE__, __LINE__);
 				break;
 			}
 			if (count < 0) {
 				int err = errno;
-				log(3, "read error: %s", strerror(err));
+				jlog(3, "read error: %s", strerror(err));
 				set_errstr(strerror(err));
 				/* comm */
 				sret = 1;
@@ -637,7 +648,7 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 				sigaddset(&sigset, SIGCHLD);
 				ret = sigprocmask(SIG_BLOCK, &sigset, &oldset);
 				if (ret < 0) {
-					log(3, "sigprocmask() error: %s",
+					jlog(3, "sigprocmask() error: %s",
 						strerror(errno));
 				}
 
@@ -650,7 +661,7 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 				ret = sigprocmask(SIG_UNBLOCK, &sigset,
 						&oldset);
 				if (ret < 0) {
-					log(3, "sigprocmask() error releasing "
+					jlog(3, "sigprocmask() error releasing "
 						"the blocked signals: %s",
 						strerror(errno));
 				}
@@ -665,18 +676,22 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 					cachewritten = write(clntinfo->cachefd,
 						pbuf, count);
 					if (cachewritten != count) {
-						log(3, "Error writing to the "
+						jlog(3, "Error writing to the "
 							"cache: %s",
 							strerror(errno));
 						cachefail = 1;
 					}
 				}
 				/* convert if we have to */
-				if (clntinfo->havetoconvert != CONV_NOTCONVERT) {
+				if (clntinfo->transfermode_havetoconvert
+							!= CONV_NOTCONVERT
+				    && clntinfo->serverlisting != 1) {
 					char* tmp;
-					if (clntinfo->havetoconvert
+					if (clntinfo->transfermode_havetoconvert
 							== CONV_TOASCII) {
-						tmp = to_ascii(buffer, &count);
+						jlog(9, "Converting to ASCII");
+						tmp = to_ascii(buffer, &count,
+							strictasciiconversion);
 						if (count > TRANSMITBUFSIZE) {
 							buffer =
 							realloc(buffer, count);
@@ -685,23 +700,25 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 							(void*) tmp, count);
 						free(tmp);
 					}
-					if (clntinfo->havetoconvert
+					if (clntinfo->transfermode_havetoconvert
 							== CONV_FRMASCII) {
-						tmp = from_ascii(buffer,
-								&count);
-						buffer = tmp;
+						/* we don't convert from
+						 * ascii, this case does not
+						 * occur, jftpgw always
+						 * reads in binary mode */
 					}
+					pbuf = buffer;
 				}
 				nwritten = write(clntinfo->dataclientsock,
 						pbuf, count);
 				if (nwritten < 0) {
-					log(2, "Error writing (%s, %d): %s",
+					jlog(2, "Error writing (%s, %d): %s",
 							__FILE__, __LINE__,
 							strerror(errno));
 					break;
 				}
 				if (nwritten == 0) {
-					log(8, "Wrote 0 bytes (%s, %d)",
+					jlog(8, "Wrote 0 bytes (%s, %d)",
 							__FILE__, __LINE__);
 				}
 				totwritten += nwritten;
@@ -760,35 +777,35 @@ int transfer_transmit(struct clientinfo *clntinfo)  {
 	if (!aborted) {
 		ret = fcntl(clntinfo->dataclientsock, F_SETFL, fdflags);
 		if (ret < 0) {
-			log(2, "Error resetting fd flags");
+			jlog(2, "Error resetting fd flags");
 		}
 	}
 
 	if (sret == -1 || scret == -1) {
-		log(2, "Error in select() occured: %s", strerror(errno));
+		jlog(2, "Error in select() occured: %s", strerror(errno));
 		lcs.respcode = 500;
 		say(cs, "500 Error in select() occured\r\n");
 		error = TRNSMT_NOERRORMSG;
 	}
 	/* count == 0 means correct end of transfer */
 	if ((sret == 0 || scret == 0) && count != 0) {
-		log(2, "Connection timed out in transfer_transmit(): %d, %d",
+		jlog(2, "Connection timed out in transfer_transmit(): %d, %d",
 				readtime.tv_sec, writetime.tv_sec);
-		log(9, "Sockets: dataserver: %d, dataclient: %d, client: %d",
+		jlog(9, "Sockets: dataserver: %d, dataclient: %d, client: %d",
 				clntinfo->dataserversock,
 				clntinfo->dataclientsock,
 				clntinfo->clientsocket);
 		if (scret == 0) {
-			log(9, "scret == 0 (write)");
+			jlog(9, "scret == 0 (write)");
 		}
 		if (sret == 0) {
-			log(9, "sret == 0 (read)");
+			jlog(9, "sret == 0 (read)");
 		}
 		error = TRNSMT_ERROR;
 	}
 
 	lcs.transferred = totwritten;
-	log(7, "Transferred %d bytes", totwritten);
+	jlog(7, "Transferred %d bytes", totwritten);
 
 	free(buffer);
 
@@ -832,8 +849,8 @@ int checkforabort(struct clientinfo* clntinfo) {
 		return 0;
 	}
 	if (ret == 0) {
-		log(8, "Read 0 bytes at %s, %d", __FILE__, __LINE__);
-		log(8, "Read from clientfd, closing it! Should not happen!");
+		jlog(8, "Read 0 bytes at %s, %d", __FILE__, __LINE__);
+		jlog(8, "Read from clientfd, closing it! Should not happen!");
 		close(clientfd);
 		return 0;
 	}
@@ -856,8 +873,8 @@ int checkforabort(struct clientinfo* clntinfo) {
 		return 0;
 	}
 	if (ret == 0) {
-		log(8, "Read 0 bytes at %s, %d", __FILE__, __LINE__);
-		log(8, "Read from clientfd, closing it! Should not happen!");
+		jlog(8, "Read 0 bytes at %s, %d", __FILE__, __LINE__);
+		jlog(8, "Read from clientfd, closing it! Should not happen!");
 		close(clientfd);
 		return 0;
 	}
@@ -892,7 +909,7 @@ int checkforabort(struct clientinfo* clntinfo) {
 	if (send(serverfd, sabseq, 3, MSG_OOB) != 3
 		|| send(serverfd, dmseq, 1, 0) != 1
 		|| send(serverfd, "ABOR\r\n", 6, 0) != 6) {
-		log(3, "send() error: %s", strerror(errno));
+		jlog(3, "send() error: %s", strerror(errno));
 	}
 
 
@@ -918,11 +935,11 @@ int checkforabort(struct clientinfo* clntinfo) {
 			lcs.respcode = ret;
 			/* fallthrough */
 		case 226:
-			log(6, "Client requested ABORT - Aborted successfully");
+			jlog(6, "Client requested ABORT - Aborted successfully");
 			free(buf);
 			return 0;
 		default:
-			log(6, "Answer of server after ABORT unknown: (%s).",
+			jlog(6, "Answer of server after ABORT unknown: (%s).",
 					buf);
 	}
 	free(buf);
