@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -83,6 +83,7 @@ struct conf_dat configuration_data[] = {
 	{"transparent-forward-include-port", TAG_CONNECTED, "on", EM, WSP },
 	{"transparent-proxy",	TAG_CONNECTED, "off", EM, WSP },
 	{"logintime",		TAG_TO | TAG_CONNECTED, "user", EM, WSP },
+	{"loginprotocolviolations",	TAG_ALL, "10", EM, WSP },
 	{"forward",		TAG_ALL, (char*) 0, EM, FL },
 	{"udpport",		TAG_CONNECTED, "2370", EM, FL },
 	{"getinternalip",	TAG_CONNECTED, "udp", EM, FL },
@@ -95,13 +96,13 @@ struct conf_dat configuration_data[] = {
 	{"logfile",		TAG_ALL, "/var/log/jftpgw.log", EM, WSP },
 	{"syslogfacility",	TAG_ALL, "daemon", EM, WSP },
 	{"cmdlogfile",		TAG_ALL, (char*) 0, EM, WSP },
-	{"cmdlogfile-style",	TAG_ALL, (char*) 0, EM, WSP },
-	{"cmdlogfile-specs",	TAG_ALL, (char*) 0, EM, WSP },
+	{"cmdlogfile-style",	TAG_ALL, (char*) 0, EM, FL },
+	{"cmdlogfile-specs",	TAG_ALL, (char*) 0, EM, FL },
 	{"connectionlogdir",	TAG_ALL, (char*) 0, EM, WSP },
 	{"connectionlogdir-filesuffix",	TAG_ALL, (char*) 0, EM, WSP },
 	{"connectionlogdir-fileprefix",	TAG_ALL, (char*) 0, EM, WSP },
-	{"connectionlogdir-style",	TAG_ALL, (char*) 0, EM, WSP },
-	{"connectionlogdir-specs",	TAG_ALL, (char*) 0, EM, WSP },
+	{"connectionlogdir-style",	TAG_ALL, (char*) 0, EM, FL },
+	{"connectionlogdir-specs",	TAG_ALL, (char*) 0, EM, FL },
 	{"cacheprefix",			TAG_ALL, (char*) 0, EM, WSP },
 	{"cache",			TAG_ALL,  "off"   , EM, WSP },
 	{"cachemaxsize",		TAG_ALL, "unlimited", EM, WSP },
@@ -118,11 +119,13 @@ struct conf_dat configuration_data[] = {
 	{"activeportrangeserver",	TAG_ALL, (char*) 0, EM, FL },
 	{"passiveportrangeserver",	TAG_ALL, (char*) 0, EM, FL },
 	{"defaultmode",			TAG_ALL, "asclient", EM, WSP },
+	{"strictasciiconversion",	TAG_ALL, "on", EM, WSP },
 	{"allowreservedports",		TAG_ALL, "no", EM, WSP },
 	{"allowforeignaddress",		TAG_ALL, "no", EM, WSP },
 	{"serverport",			TAG_ALL, "21", EM, WSP },
 	{"loginstyle",			TAG_CONNECTED, "1", EM, WSP },
 	{"account",			TAG_CONNECTED, (char*) 0, EM, FL },
+	{"initialsyst",			TAG_ALL, "yes", EM, WSP },
 	{"commandtimeout",		TAG_ALL, "300", EM, WSP },
 	{"transfertimeout",		TAG_ALL, "300", EM, WSP },
 	{"reverselookups",		TAG_ALL, "yes", EM, WSP },
@@ -153,7 +156,7 @@ struct conf_opt configuration_options[] = {
 	{"logintime",           {"connect", "user", "pass", TERM} },
 	{"access",              {"allow", "deny", TERM} },
 	{"defaultmode",         {"asclient", "active", "passive", TERM} },
-	{"getinternalip",       {"udp", "icmp", TERM} },
+	{"getinternalip",       {"udp", "icmp", "configuration", TERM} },
 	{"transparent-proxy",   { TRUEFALSE, TERM } },
 	{"cache",               { TRUEFALSE, TERM } },
 	{"allowreservedports",  { TRUEFALSE, TERM } },
@@ -675,9 +678,9 @@ struct hostent_list* hostent_init(struct hostent* e,
 	enough_mem(h);
 
 	if (name) {
-		/* log(9, "adding %s to host cache", name); */
+		/* jlog(9, "adding %s to host cache", name); */
 	} else {
-		/* log(9, "adding %s to host cache",
+		/* jlog(9, "adding %s to host cache",
 				inet_ntoa(*((struct in_addr*) &ip))); */
 	}
 
@@ -849,7 +852,7 @@ struct hostlist_t* hostlist_push(struct hostlist_t** hl,
 	struct hostlist_t* h;
 	if (! hl) {
 		/* this should NOT happen */
-		log(3, "hostlist was NULL in hostlist_push()");
+		jlog(3, "hostlist was NULL in hostlist_push()");
 		return (struct hostlist_t*) 0;
 	}
 	h = *hl;
@@ -948,6 +951,16 @@ struct portrangestruct* portrangestruct_clone(struct portrangestruct* prs) {
 /* --------end struct portrangestruct---------------- */
 
 
+/* returns the first characters up to a whitespace character */
+
+/* Checks if PATTERN is the beginning of RESPONSE */
+
+int checkbegin(const char* response, const char* pattern) {
+	if (strlen(response) < strlen(pattern)) {
+		return 0;
+	}
+	return !strncasecmp(response, pattern, strlen(pattern));
+}
 
 char* config_read_line_basic(FILE* file) {
 	/* read 255 bytes at first */
@@ -966,7 +979,7 @@ char* config_read_line_basic(FILE* file) {
 	times = 0;
 	do {
 		if (size > INT_MAX - increase) {
-			log(1, "input in config file too long");
+			jlog(1, "input in config file too long");
 			free(line);
 			line = (char*) 0;
 			return (char*) 0;
@@ -987,7 +1000,7 @@ char* config_read_line_basic(FILE* file) {
 			}
 			perror("Error reading a line from the "
 			       "configuration file");
-			log(1, "Error reading a line from the "
+			jlog(1, "Error reading a line from the "
 			       "configuration file: %s", strerror(errno));
 		}
 		size += increase;
@@ -1089,7 +1102,11 @@ char* quotstrtok(const char* s, const char* delim, int *past_offset) {
 		return 0;
 	if (s[i] == '"') {
 		q = strchr(&s[i+1], '"');
-		q = strpbrk(++q, delim);
+		if (q) {
+			q = strpbrk(++q, delim);
+		} else {
+			q = strpbrk(&s[i], delim);
+		}
 	} else {
 		q = strpbrk(&s[i], delim);
 	}
@@ -1326,8 +1343,7 @@ static
 const char* config_get_delimiter(const char* key) {
 	int i = 0;
 	while (configuration_data[i].name) {
-		if (strncasecmp(key, configuration_data[i].name,
-				strlen(configuration_data[i].name)) == 0) {
+		if (strcasecmp(key, configuration_data[i].name) == 0) {
 			/* found */
 			if (configuration_data[i].read_limit == FL) {
 				return "\n";
@@ -1357,13 +1373,13 @@ int config_parse_option(const char* line, int tag_num,
 	free(tmp);
 
 	if ( ! key || ! value) {
-		log(4, "Could not parse option %s", line);
+		jlog(4, "Could not parse option %s", line);
 		free(key); free(value);
 		return -1;
 	}
 
 	if ( ! strlen(key) || ! strlen(value) ) {
-		log(4, "Could not parse option %s", line);
+		jlog(4, "Could not parse option %s", line);
 		free(key); free(value);
 		return -1;
 	}
@@ -1371,10 +1387,10 @@ int config_parse_option(const char* line, int tag_num,
 	if ( ! config_is_option_key_valid(key, tag_num) ) {
 		char* tags = config_get_valid_tags(key);
 		if (tags) {
-			log(5, "%s only valid in those tags:%s", key, tags);
+			jlog(5, "%s only valid in those tags:%s", key, tags);
 			free(tags);
 		} else {
-			log(5, "unknown option: %s", key);
+			jlog(5, "unknown option: %s", key);
 		}
 		free(key); free(value);
 		return -1;
@@ -1383,7 +1399,7 @@ int config_parse_option(const char* line, int tag_num,
 	if ( ! config_is_option_value_valid(key, value) ) {
 		char* values = config_get_valid_values(key);
 		if (values) {
-			log(5, "%s may only take those values: %s",
+			jlog(5, "%s may only take those values: %s",
 								key, values);
 			free(values);
 			return -1;
@@ -1439,7 +1455,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 		offset = 0;
 		startportstr = quotstrtok(slistcur->value, ":", &offset);
 		if (!startportstr) {
-			log(3, "invalid port range: %s", slistcur->value);
+			jlog(3, "invalid port range: %s", slistcur->value);
 			slist_destroy(slistbase);
 			return (struct portrangestruct*) 0;
 		}
@@ -1459,7 +1475,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 					&offset);
 		}
 		if (!*startportstr || !*endportstr) {
-			log(3, "invalid port range: %s", slistcur->value);
+			jlog(3, "invalid port range: %s", slistcur->value);
 			if (startportstr) { free(startportstr); }
 			if (endportstr)   { free(endportstr);   }
 			slist_destroy(slistbase);
@@ -1468,7 +1484,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 		startport = strtol(startportstr, NULL, 10);
 		if (errno == ERANGE
 				&& (startport == LONG_MIN || startport == LONG_MAX)) {
-			log(3, "Error reading the starting port number in %s",
+			jlog(3, "Error reading the starting port number in %s",
 				startportstr);
 			slist_destroy(slistbase);
 			free(startportstr);
@@ -1481,7 +1497,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 		endport = strtol(endportstr, NULL, 10);
 		if (errno == ERANGE
 				&& (endport == LONG_MIN || endport == LONG_MAX)) {
-			log(3, "Error reading the ending port number in %s",
+			jlog(3, "Error reading the ending port number in %s",
 				endportstr);
 			slist_destroy(slistbase);
 			free(endportstr);
@@ -1492,7 +1508,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 
 		/* startport really below or equal to endport ? */
 		if (startport > endport) {
-			log(4, "Port range %d:%d invalid (starting port number"
+			jlog(4, "Port range %d:%d invalid (starting port number"
 				" %d is _above_ endport %d)",
 					startport, endport,
 					startport, endport);
@@ -1543,7 +1559,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 			    (endport >= prs->startport &&
 			     endport <= prs->endport)
 			   ) {
-				log(4, "Port range %d:%d invalid (parts "
+				jlog(4, "Port range %d:%d invalid (parts "
 					"covered by another range (%d:%d))",
 						startport, endport,
 						prs->startport, prs->endport);
@@ -1558,7 +1574,7 @@ struct portrangestruct* config_parse_portranges(const char* line) {
 			    (prs->endport >= startport &&
 			     prs->endport <= endport)
 			    ) {
-				log(4, "Port range %d:%d invalid (parts "
+				jlog(4, "Port range %d:%d invalid (parts "
 					"covered by another range (%d:%d))",
 						prs->startport, prs->endport,
 						startport, endport);
@@ -1645,11 +1661,11 @@ struct hostlist_t* config_parse_ips(struct hostlist_t** hl,
 				p++;
 			}
 			ipbuf[i] = '\0';
-			/* log(9, "Found ip %s", ipbuf); */
+			/* jlog(9, "Found ip %s", ipbuf); */
 			ip = inet_addr(ipbuf);
 			if (ip == (unsigned long int) UINT_MAX
 					&& strcmp(ipbuf, BROADCAST)) {
-				log(4, "Invalid IP: %s", ipbuf);
+				jlog(4, "Invalid IP: %s", ipbuf);
 				return (struct hostlist_t *) 0;
 			}
 			if (*p == '/') {
@@ -1661,7 +1677,7 @@ struct hostlist_t* config_parse_ips(struct hostlist_t** hl,
 					p++;
 				}
 				ipbuf[i] = '\0';
-			/*	log(8, "With netmask %s", ipbuf); */
+			/*	jlog(8, "With netmask %s", ipbuf); */
 				invalid = 0;
 
 				if (strlen(ipbuf) < 3 && !strchr(ipbuf, '.')) {
@@ -1681,7 +1697,7 @@ struct hostlist_t* config_parse_ips(struct hostlist_t** hl,
 					}
 				}
 				if (invalid) {
-					log(4, "Invalid netmask: %s", ipbuf);
+					jlog(4, "Invalid netmask: %s", ipbuf);
 
 					/* Set the netmask to
 					 * 255.255.255.255 */
@@ -1746,7 +1762,7 @@ struct hostlist_t* config_parse_names(struct hostlist_t** hl,
 			free(tmp);
 			tmp = 0;
 		}
-		/* log(9, "Read name %s", buf); */
+		/* jlog(9, "Read name %s", buf); */
 
 		h_list = hostlist_name_push(hl, buf);
 	}
@@ -1832,7 +1848,7 @@ int config_parse_time(char* line, struct timestruct** ts) {
 			offset3 = 0;
 			while ((wday = quotstrtok(dat_str, "/", &offset3))) {
 				if (day_number(wday) < 0) {
-					log(5, "Invalid date (day %s not known): %s", wday, entry);
+					jlog(5, "Invalid date (day %s not known): %s", wday, entry);
 					free(wday); free(dat_str); free(entry);
 					/* set dat_str to 0 to indicate that
 					 * we this date is invalid */
@@ -1862,17 +1878,17 @@ int config_parse_time(char* line, struct timestruct** ts) {
 					&t->start_hour, &t->start_minute);
 		}
 		if (i != 2) {
-			log(5, "Illegal start time: %s", entry);
+			jlog(5, "Illegal start time: %s", entry);
 			free(entry); free(tm_str);
 			break;
 		}
 		if (t->start_hour < 0 || t->start_hour > 23) {
-			log(5, "Illegal start time: %s", entry);
+			jlog(5, "Illegal start time: %s", entry);
 			free(entry); free(tm_str);
 			break;
 		}
 		if (t->start_minute < 0 || t->start_minute > 59) {
-			log(5, "Illegal start time: %s", entry);
+			jlog(5, "Illegal start time: %s", entry);
 			free(entry); free(tm_str);
 			break;
 		}
@@ -1893,7 +1909,7 @@ int config_parse_time(char* line, struct timestruct** ts) {
 			free(dat_str);
 			/* was a date */
 			if (t->days) {
-				log(5, "You can't specify a day for the end time if you specified a range of days for the start time: %s", entry);
+				jlog(5, "You can't specify a day for the end time if you specified a range of days for the start time: %s", entry);
 				free(dat_str); free(entry);
 				break;
 			}
@@ -1901,7 +1917,7 @@ int config_parse_time(char* line, struct timestruct** ts) {
 		} else {
 			/* may be a time */
 			if (strchr(dat_str, '/')) {
-				log(5, "Ranges not allowed for the end time: %s", entry);
+				jlog(5, "Ranges not allowed for the end time: %s", entry);
 				free(dat_str); free(entry);
 				break;
 			}
@@ -1912,7 +1928,7 @@ int config_parse_time(char* line, struct timestruct** ts) {
 						&t->end_hour, &t->end_minute);
 		}
 		if (i != 2) {
-			log(5, "Illegal end time: %s", entry);
+			jlog(5, "Illegal end time: %s", entry);
 			free(dat_str); free(entry);
 			break;
 		}
@@ -1926,7 +1942,7 @@ int config_parse_time(char* line, struct timestruct** ts) {
 			 * times make sense */
 			if (t->end_hour*100 + t->end_minute <
 			    t->start_hour*100 + t->start_minute) {
-				log(5, "end time before start time on the same day: %s", entry);
+				jlog(5, "end time before start time on the same day: %s", entry);
 				free(entry);
 				break;
 			}
@@ -1991,7 +2007,7 @@ struct tag_options_t config_parse_tag_options(char* line) {
 		enough_mem(to.list_str);
 
 		if (! exclude) {
-			log(4, "malformed line starting with %s", space);
+			jlog(4, "malformed line starting with %s", space);
 			free(to.list_str);
 			to.list_str = (char*) 0;
 			return to;
@@ -2269,7 +2285,7 @@ struct section_t* config_parse_section(FILE* file,
 			}
 			tag_name = config_get_tag_name(line, tag_type);
 			if (tag_name == TAG_UNKNOWN) {
-				log(3, "invalid tag: %s", line);
+				jlog(3, "invalid tag: %s", line);
 				config_error = 1;
 				return (struct section_t*) 0;
 			}
@@ -2286,6 +2302,10 @@ struct section_t* config_parse_section(FILE* file,
 
 				free(recline);
 
+				if (!s) {
+					break;
+				}
+
 				if (! section->nested) {
 					section->nested = s;
 				} else {
@@ -2301,14 +2321,14 @@ struct section_t* config_parse_section(FILE* file,
 				/* tag_type = TAG_CLOSING */
 				int last_opened;
 				if (ilist_empty(tag_list)) {
-					log(3, "Closed a tag where no one "
+					jlog(3, "Closed a tag where no one "
 							"was open: %s", line);
 					config_error = 1;
 					return (struct section_t*) 0;
 				}
 				last_opened = ilist_pop(tag_list);
 				if (last_opened != tag_name) {
-					log(3, "Closed a tag that was not "
+					jlog(3, "Closed a tag that was not "
 						"opened before: %s", line);
 					config_error = 1;
 					return (struct section_t*) 0;
@@ -2441,7 +2461,7 @@ FILE* open_file(const char* fname) {
 	conf = fopen(fname, "r");
 	if (!conf) {
 		perror("Couldn't open the configuration file");
-		log(1, "Couldn't open the configuration file %s: %s", fname,
+		jlog(1, "Couldn't open the configuration file %s: %s", fname,
 		strerror(errno));
 		return 0;
 	}
@@ -2466,7 +2486,7 @@ int config_read_sections(FILE* file) {
 		do {
 			line = config_read_line(file);
 			if ( line && line[0] != '<' ) {
-				log(6, "Garbage in config file: %s", line);
+				jlog(6, "Garbage in config file: %s", line);
 			}
 		} while (line && (line[0] != '<' || line[1] == '/'));
 
@@ -2478,7 +2498,7 @@ int config_read_sections(FILE* file) {
 		if ( counter == 1 && ! strncasecmp(&line[1],
 				"global", strlen("global")) == 0) {
 
-			log(4, "global section is not the first one. "
+			jlog(4, "global section is not the first one. "
 				"Exiting.");
 			return -1;
 		}
@@ -2487,7 +2507,7 @@ int config_read_sections(FILE* file) {
 		}
 		tag_name = config_get_tag_name(line, TAG_OPENING);
 		if (tag_name == TAG_UNKNOWN) {
-			log(5, "Invalid tag: %s", line);
+			jlog(5, "Invalid tag: %s", line);
 			continue;
 		}
 		ilist_push(tag_list, tag_name);
@@ -2525,12 +2545,12 @@ int config_read_sections(FILE* file) {
 	}
 
 	if (! base_section) {
-		log(3, "No global section in configuration file");
+		jlog(3, "No global section in configuration file");
 		return -1;
 	}
 
 	if ( ! ilist_empty(tag_list)) {
-		log(3, "Not all tags have been closed");
+		jlog(3, "Not all tags have been closed");
 		return -1;
 	}
 
@@ -2555,7 +2575,7 @@ int read_config(const char* fname) {
 		free(filename);
 		return -1;
 	}
-	log(9, "opened file %s", filename);
+	jlog(9, "opened file %s", filename);
 	free(filename);
 
 	ret = config_read_sections(conffile);
@@ -2568,7 +2588,7 @@ int read_config(const char* fname) {
 			return -1;
 		}
 		/* backup successfully activated */
-		log(2, "New configuration NOT active - backup configuration"
+		jlog(2, "New configuration NOT active - backup configuration"
 				" has been re-enabled");
 	} else {
 		/* destroy the backup and create a new copy, we've read a
@@ -2700,7 +2720,7 @@ int config_activate_backup() {
 	backup_base_section = (struct section_t*) 0;
 	config_create_backup();
 
-	log(8, "Backup configuration activated");
+	jlog(8, "Backup configuration activated");
 
 	return 0;
 }
@@ -2736,15 +2756,15 @@ const struct hostent_list* config_forward_lookup(struct hostent_list** hl,
 		return (struct hostent_list*) 0;
 	}
 
-	/* log(9, "looking up %s", name); */
+	/* jlog(9, "looking up %s", name); */
 	if (*hl) {
 		h = hostent_get(*hl, -1, name);
 		if (config_host_valid(h)) {
-		/*	log(9, "Could use cache for forward lookup of %s",
+		/*	jlog(9, "Could use cache for forward lookup of %s",
 					name); */
 			return h;
 		} else {
-		/*	log(9, "Could not use cache for forward lookup of %s",
+		/*	jlog(9, "Could not use cache for forward lookup of %s",
 					name); */
 			hostent_delete(*hl, h);
 		}
@@ -2778,17 +2798,17 @@ const struct hostent_list* config_reverse_lookup(struct hostent_list** hl,
 		return (struct hostent_list*) 0;
 	}
 
-	/* log(9, "looking up %s", inet_ntoa(*((struct in_addr*) &ip))); */
+	/* jlog(9, "looking up %s", inet_ntoa(*((struct in_addr*) &ip))); */
 	if (*hl) {
 		h = hostent_get(*hl, ip, (char*) 0);
 		if (config_host_valid(h)) {
-		/*	log(9, "Could use cache for reverse lookup of %s",
+		/*	jlog(9, "Could use cache for reverse lookup of %s",
 					inet_ntoa(*((struct in_addr*) &ip)));
 		*/
 			return h;
 		} else {
 			hostent_delete(*hl, h);
-		/*	log(9, "Could not use cache for reverse lookup of %s",
+		/*	jlog(9, "Could not use cache for reverse lookup of %s",
 					inet_ntoa(*((struct in_addr*) &ip)));
 		*/
 		}
@@ -2908,7 +2928,7 @@ int match_addrs(unsigned long int ip, const char* name,
 			 * ipwnet/netmask */
 			host = config_forward_lookup(hl, name);
 			if (!host) {
-				log(5, "Could not look up (1) name %s", name);
+				jlog(5, "Could not look up (1) name %s", name);
 			} else {
 				/* see if one alias of the host matches the
 				 * allowed pattern */
@@ -2947,7 +2967,7 @@ int match_addrs(unsigned long int ip, const char* name,
 		host = config_reverse_lookup(hl, ip);
 		if (!host) {
 			iaddr.s_addr = ip;
-			log(5, "Could not look up IP %s", inet_ntoa(iaddr));
+			jlog(5, "Could not look up IP %s", inet_ntoa(iaddr));
 		} else {
 			/* see if one IP of the host is allowed by the
 			 * IP/netmask fields */
@@ -2966,7 +2986,7 @@ int match_addrs(unsigned long int ip, const char* name,
 		if (namepat) {
 			host = config_forward_lookup(hl, namepat);
 			if (!host) {
-				log(5, "Could not look up (2) name %s",
+				jlog(5, "Could not look up (2) name %s",
 						namepat);
 			} else {
 				/* see if one IP of the host is allowed by the
@@ -3416,7 +3436,7 @@ int config_shrink_config(unsigned long int from_ip,
 		 * getsockname */
 		/* ???????
 		 * config_state = TAG_GLOBAL | TAG_FROM | TAG_TO | TAG_PORT;
-		 * log(9, "Checking TAG_GLOBAL | TAG_FROM | TAG_TO | TAG_PORT");		 */
+		 * jlog(9, "Checking TAG_GLOBAL | TAG_FROM | TAG_TO | TAG_PORT");		 */
 	config_match_section(base_section, from_ip,
 			to_ip, to_name, to_port, to_user,
 			forw_ip, forw_name, forw_port, forw_user,
@@ -3523,7 +3543,7 @@ const char* config_get_default_value(const char* key) {
 		if (strcasecmp(configuration_data[i].name, key) == 0) {
 			val = configuration_data[i].defaultvalue;
 			if (val) {
-				log(8, "Did not find configuration "
+				jlog(8, "Did not find configuration "
 					"entry for \"%s\", using "
 					"\"%s\" as default", key, val);
 				/* insert into option list */
@@ -3634,7 +3654,7 @@ unsigned long config_get_addroption(const char* key, unsigned long err_return){
 		/* IP was recognized */
 		return sin.sin_addr.s_addr;
 	}
-	log(6, "Invalid IP/interface: %s", optstr);
+	jlog(6, "Invalid IP/interface: %s", optstr);
 	return err_return;
 }
 
@@ -3662,6 +3682,10 @@ float config_get_foption(const char* key, float err_return) {
 int config_get_bool(const char* key) {
 	const char* optstr = config_get_option(key);
 
+	if (!optstr) {
+		jlog(5, "There was no value for %s, using \"off\"", key);
+		return 0;
+	}
 	if (strcasecmp(optstr, "on") == 0) {
 		return 1;
 	}
@@ -3687,7 +3711,7 @@ int config_get_bool(const char* key) {
 		return 0;
 	}
 
-	log(5, "There was no value for %s, using \"off\"", key);
+	jlog(5, "There was no value for %s, using \"off\"", key);
 	return 0;
 }
 
@@ -3716,7 +3740,7 @@ unsigned long int config_get_size(const char* key,
 
 	i = sscanf(optstr, "%lu%c", &size, &multiplier);
 	if (i != 2) {
-		log(5, "%s not a valid size", optstr);
+		jlog(5, "%s not a valid size", optstr);
 		return err_return;
 	}
 	multiplier = toupper((int) multiplier);
@@ -3730,7 +3754,7 @@ unsigned long int config_get_size(const char* key,
 		case 'G':
 			return size * 1024 * 1024 * 1024;
 		default:
-			log(5, "%s not a valid size", optstr);
+			jlog(5, "%s not a valid size", optstr);
 	}
 	return err_return;
 }
