@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+/*
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -54,7 +54,7 @@ int fw_validate(const char* fwuser, const char* fwpass) {
 	const char* user, *method, *pass;
 
 	if ( ! acct_list ) {
-		log(5, "No account information found for %s", fwuser);
+		jlog(5, "No account information found for %s", fwuser);
 		return 1;
 	}
 
@@ -96,7 +96,7 @@ int fw_validate(const char* fwuser, const char* fwpass) {
 		continue;
 error_line:
 		slist_destroy(acct_line);
-		log(5, "Incorrect account specification: %s", account->value);
+		jlog(5, "Incorrect account specification: %s", account->value);
 	} while ( (account = account -> next) );
 
 	slist_destroy(acct_list);
@@ -109,7 +109,7 @@ int std_reset(const char* args, struct conn_info_st* conn_info) {
 	/* set all values to (char*) 0 after free()ing */
 	struct clientinfo* c = conn_info->clntinfo;
 
-	log(8, "resetting login information");
+	jlog(8, "resetting login information");
 
 	if (!c->transparent == TRANSPARENT_YES ||
 	    !config_compare_option("logintime", "connect")) {
@@ -193,7 +193,7 @@ int fw_forward(const char* user,
 
 	if ( ! user ) {
 		say(clntinfo->clientsocket, "500 Expecting user name\r\n");
-		log(5, "No username given");
+		jlog(5, "No username given");
 		return CMD_ERROR;
 	}
 
@@ -204,7 +204,7 @@ int fw_forward(const char* user,
 
 	if (strcmp(user, "*") == 0) {
 		/* keep the old user name */
-		log(8, "no new user name - keeping old one: %s",
+		jlog(8, "no new user name - keeping old one: %s",
 						clntinfo->user);
 	} else {
 		free(clntinfo->user);
@@ -214,7 +214,7 @@ int fw_forward(const char* user,
 
 	if (!dest || strlen(dest) == 0 || strcmp(dest, "*") == 0) {
 		/* keep the old one */
-		log(8, "no new destination - keeping old one: %s",
+		jlog(8, "no new destination - keeping old one: %s",
 					clntinfo->destination);
 	} else {
 		free(clntinfo->destination);
@@ -234,23 +234,23 @@ int fw_transparent(struct clientinfo* clntinfo) {
 
 	if (clntinfo->transparent == TRANSPARENT_NO) {
 		/* not in transparent mode */
-		log(9, "fw_transparent: not in transparent mode");
+		jlog(9, "fw_transparent: not in transparent mode");
 		return CMD_HANDLED;
 	}
 
 	if (transfor_opt) {
 		/* transparent forward active */
-		log(9, "fw_transparent: transparent forward active");
+		jlog(9, "fw_transparent: transparent forward active");
 		return CMD_HANDLED;
 	}
 
 	if (clntinfo->destination) {
-		log(9, "fw_transparent: Destination already set");
+		jlog(9, "fw_transparent: Destination already set");
 		return CMD_HANDLED;
 	}
 
 	if (config_get_bool("transparent-proxy") == 0) {
-		log(4, "No destination set and transparent proxy support disabled");
+		jlog(4, "No destination set and transparent proxy support disabled");
 		say(clntinfo->clientsocket, "500 No destination\r\n");
 		return CMD_ERROR;
 	}
@@ -260,10 +260,16 @@ int fw_transparent(struct clientinfo* clntinfo) {
 	     ntohs(clntinfo->transparent_destination.sin_port);
 
 	/* Determine the IP the client sees of us */
-	clntinfo->addr_to_client =
+	if (strcasecmp(config_get_option("getinternalip"),
+						"configuration") == 0) {
+		clntinfo->addr_to_client =
+			config_get_addroption("dataclientaddress", INADDR_ANY);
+	} else {
+		clntinfo->addr_to_client =
 		socketinfo_get_local_addr_by_sending(clntinfo->clientsocket);
+	}
 
-	log(8, "Using transparent proxy. Connecting to %s port %d. User: %s",
+	jlog(8, "Using transparent proxy. Connecting to %s port %d. User: %s",
 				clntinfo->destination,
 				clntinfo->destinationport,
 				clntinfo->user);
@@ -292,7 +298,7 @@ int fw_transparent_forward(struct clientinfo* clntinfo) {
 	transparent_target = socketinfo_get_transparent_target_char(
 					clntinfo->clientsocket);
 	if (!transparent_target) {
-		log(4, "Could not get transparent destination");
+		jlog(4, "Could not get transparent destination");
 		say(clntinfo->clientsocket, "500 Error logging in\r\n");
 		return CMD_ERROR;
 	}
@@ -342,7 +348,7 @@ int fw_transparent_forward(struct clientinfo* clntinfo) {
 		if (! destination.hostname) {
 			say(clntinfo->clientsocket,
 						"500 Error logging in\r\n");
-			log(4, "Could not parse transparent forward target\r\n");
+			jlog(4, "Could not parse transparent forward target\r\n");
 			return CMD_ERROR;
 		}
 		free(clntinfo->destination);
@@ -354,12 +360,19 @@ int fw_transparent_forward(struct clientinfo* clntinfo) {
 	}
 
 	/* Determine the IP the client sees of us */
-	clntinfo->addr_to_client =
+	if (strcasecmp(config_get_option("getinternalip"),
+						"configuration") == 0) {
+		clntinfo->addr_to_client =
+			config_get_addroption("dataclientaddress", INADDR_ANY);
+	} else {
+		clntinfo->addr_to_client =
 		socketinfo_get_local_addr_by_sending(clntinfo->clientsocket);
+	}
 	clntinfo->transparent = TRANSPARENT_YES;
 
-	log(8, "Using transparent proxy. Connecting to %s. User: %s",
+	jlog(8, "Using transparent forward. Connecting to %s port %d. User: %s",
 					clntinfo->destination,
+					clntinfo->destinationport,
 					clntinfo->user);
 
 	return CMD_HANDLED;
@@ -393,15 +406,15 @@ int fw_port_mode(const char* portstr,
 	clntinfo->servermode = UNSPEC;
 	if (modestr) {
 		if (modestr && (strchr("ap", *modestr))) {
-			log(9, "mode specified: %s", modestr);
+			jlog(9, "mode specified: %s", modestr);
 			switch (*modestr) {
 				case 'a': 
 					clntinfo->servermode = ACTIVE;
-					log(9, "p-s: active");
+					jlog(9, "p-s: active");
 					break;
 				case 'p':
 					clntinfo->servermode = PASSIVE;
-					log(9, "p-s: passive");
+					jlog(9, "p-s: passive");
 					break;
 			}
 		}
@@ -476,7 +489,7 @@ int std_user(const char* args, struct conn_info_st* conn_info,
 		return CMD_ERROR;
 	}
 
-	log(7, "Client logged in: User: %s, Dest: %s:%d",
+	jlog(7, "Client logged in: User: %s, Dest: %s:%d",
 				conn_info->clntinfo->user,
 				conn_info->clntinfo->destination,
 				conn_info->clntinfo->destinationport);
@@ -522,7 +535,7 @@ int cmds_after_user(struct conn_info_st* conn_info) {
 		 * receiving the result, now check the result */
 		code = getcode(conn_info->clntinfo->login.authresp.fullmsg);
 		if (code != 331 && code != 230) {
-			log(6, "Didn't get successful message after sending "
+			jlog(6, "Didn't get successful message after sending "
 				"the user name: %s\n",
 				conn_info->clntinfo->login.authresp.fullmsg);
 			return CMD_ERROR;
@@ -564,13 +577,10 @@ int cmds_after_user(struct conn_info_st* conn_info) {
 
 int std_pass(const char* args, struct conn_info_st* conn_info) {
 
-	int offset;
-
-	offset = strlen("PASS ");
-	conn_info->clntinfo->pass = quotstrtok(args, WHITESPACES, &offset);
+	conn_info->clntinfo->pass = strdup(args + strlen("PASS "));
+	enough_mem(conn_info->clntinfo->pass);
 
 	return cmds_after_pass(conn_info);
-
 }
 
 static
@@ -670,14 +680,19 @@ int fw_user(const char* args, struct conn_info_st* conn_info) {
 static
 int fw_set_pass(const char* args, struct conn_info_st* conn_info) {
 	/* just chop off the command and put everything in the password */
-	if ( ! args || *(args + strlen("PASS ")) == '\0') {
+	if ( ! args ) {
 		say(conn_info->clntinfo->clientsocket,
 				"530-Not a valid password\r\n"
 				"530 Login failed.\r\n");
 		return CMD_ERROR;
 	}
-
-	conn_info->clntinfo->pass = strdup(args + strlen("PASS "));
+	if (*(args + strlen("PASS")) == '\0' ||
+		*(args + strlen("PASS ")) == '\0') {
+		/* allow empty passwords, too */
+		conn_info->clntinfo->pass = strdup("");
+	} else {
+		conn_info->clntinfo->pass = strdup(args + strlen("PASS "));
+	}
 	enough_mem(conn_info->clntinfo->pass);
 	return CMD_HANDLED;
 }
@@ -777,7 +792,7 @@ int fw_user_type7(const char* args, struct conn_info_st* conn_info) {
 	struct destination_t destination;
 
 	if (config_compare_option("logintime", "pass") == 0) {
-		log(4, "logintime has to be \"pass\" with loginstyle 7");
+		jlog(4, "logintime has to be \"pass\" with loginstyle 7");
 		say(conn_info->clntinfo->clientsocket,
 				"550 Login incorrect\r\n");
 		return CMD_ERROR;
