@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+/*  
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -65,7 +65,7 @@ static struct loginfo_st* loginfo_bk;
 /*
  * This routine logs messages to either the log file or the syslog function.
  */
-void log(int level, const char *fmt, ...)
+void jlog(int level, const char *fmt, ...)
 {
 	va_list args;
 	time_t nowtime;
@@ -115,7 +115,7 @@ void log(int level, const char *fmt, ...)
 			logtype = LOG_ERR;
 		}
 		vsnprintf(str, LOGSIZE - 1, fmt, args);
-		syslog(logtype, str);
+		syslog(logtype, "%s", str);
 	}
 
 	va_end(args);
@@ -179,7 +179,7 @@ const char* base_name(const char* s) {
 
 char* log_replace_char(const char pattern, struct log_cmd_st* lcs) {
 	union {
-		const char* replace_str;
+		char* replace_str;
 		char replace_char;
 		unsigned int replace_uint;
 		unsigned long int replace_luint;
@@ -194,10 +194,33 @@ char* log_replace_char(const char pattern, struct log_cmd_st* lcs) {
 			replace_val.replace_char = lcs->complete ? 'c' : 'i';
 			replace_type = LOG_REPLACE_CHAR;
 			break;
+		case 'D': /* common log time/date: [12/Feb/2003:13:34:50 +0100] */
+			{
+				time_t nowtime = time(NULL);
+				replace_val.replace_str = malloc(100);
+				enough_mem(replace_val.replace_str);
+				/* XXX %z is a GNU extension */
+				strftime(replace_val.replace_str, 100,
+					"[%d/%b/%Y:%H:%M:%S %z]",
+					localtime(&nowtime));
+				replace_type = LOG_REPLACE_STRING;
+			}
+			break;
 		case 'T': /* Time taken to transmit/receive file, in seconds */
 			replace_val.replace_uint = lcs->transfer_duration;
 			replace_type = LOG_REPLACE_UINT;
 			break;
+		case 't': /* date/time like Wed Feb 14 01:41:28 2001 */
+			{
+				time_t nowtime = time(NULL);
+				replace_val.replace_str = malloc(100);
+				enough_mem(replace_val.replace_str);
+				strftime(replace_val.replace_str, 100,
+					"%a %b %d %H:%M:%S %Y",
+					localtime(&nowtime));
+				replace_type = LOG_REPLACE_STRING;
+			}
+			break;
 		case 'b': /* Bytes sent for request */
 			replace_val.replace_luint = lcs->transferred;
 			replace_type = LOG_REPLACE_LUINT;
@@ -214,23 +237,31 @@ char* log_replace_char(const char pattern, struct log_cmd_st* lcs) {
 			}
 			break;
 		case 'f': /* Filename stored or retrieved, absolute path */
-			replace_val.replace_str = lcs->filename;
+			replace_val.replace_str = strfilldup(lcs->filename, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'F':
 			/* Filename stored or retrieved, as the client sees
 			 * it base_name is just a pointer within lcs->filename
 			 * */
-			replace_val.replace_str = base_name(lcs->filename);
+			replace_val.replace_str = strfilldup(base_name(lcs->filename), "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'm': /* Command (method) name received from client,
 			     e.g., RETR */
-			replace_val.replace_str = lcs->method;
+			replace_val.replace_str = strfilldup(lcs->method, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'r': /* full commandline */
-			replace_val.replace_str = lcs->cmd;
+			if (lcs && lcs->cmd && *(lcs->cmd) &&
+					checkbegin(lcs->cmd, "PASS")) {
+				replace_val.replace_str = strdup("PASS *");
+			} else {
+				replace_val.replace_str = strfilldup(lcs->cmd, "-");
+			}
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'P': /* pid */
@@ -251,7 +282,7 @@ char* log_replace_char(const char pattern, struct log_cmd_st* lcs) {
 			break;
 		case 'o':  /* anonymous? */
 			/* logged in ? */
-			if (!lcs || !lcs->userlogin) {
+			if (!lcs->userlogin) {
 				replace_val.replace_char = '-';
 				replace_type = LOG_REPLACE_CHAR;
 				break;
@@ -262,53 +293,97 @@ char* log_replace_char(const char pattern, struct log_cmd_st* lcs) {
 			replace_type = LOG_REPLACE_CHAR;
 			break;
 		case 'e': /* sErvice */
-			replace_val.replace_str = lcs->service;
+			replace_val.replace_str = strfilldup(lcs->service, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'n': /* aNon-user */
-			replace_val.replace_str = lcs->anon_user;
+			if (!lcs->userlogin
+				|| strcmp(lcs->userlogin, "anonymous") == 0
+				|| strcmp(lcs->userlogin, "ftp") == 0) {
+
+				replace_val.replace_str
+						= strfilldup(lcs->anon_user, "-");
+				enough_mem(replace_val.replace_str);
+			} else {
+				/* Server user name (login) */
+				replace_val.replace_str
+						= strfilldup(lcs->userlogin, "-");
+				enough_mem(replace_val.replace_str);
+			}
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'H': /* Server host name */
-			replace_val.replace_str = lcs->svrname;
+			replace_val.replace_str = strfilldup(lcs->svrname, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'A': /* Server host IP */
-			replace_val.replace_str = lcs->svrip;
+			replace_val.replace_str = strfilldup(lcs->svrip, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'd': /*  Server host name as specified in the login  */
-			replace_val.replace_str = lcs->svrlogin;
+			replace_val.replace_str = strfilldup(lcs->svrlogin, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'h': /* Client host name */
-			replace_val.replace_str = lcs->clntname;
+			replace_val.replace_str = strfilldup(lcs->clntname, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'a': /* Client host IP */
-			replace_val.replace_str = lcs->clntip;
+			replace_val.replace_str = strfilldup(lcs->clntip, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
-		case 'I': /* Server host name */
-			replace_val.replace_str = lcs->ifipsvr;
+		case 'I': /* Server interface address */
+			replace_val.replace_str = strfilldup(lcs->ifipsvr, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
-		case 'i': /* Client host IP */
-			replace_val.replace_str = lcs->ifipclnt;
+		case 'i': /* Client interface address */
+			replace_val.replace_str = strfilldup(lcs->ifipclnt, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'l': /* Server user name (login) */
-			replace_val.replace_str = lcs->userlogin;
+			replace_val.replace_str = strfilldup(lcs->userlogin, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'L': /* Effective server user name */
-			replace_val.replace_str = lcs->usereffective;
+			replace_val.replace_str = strfilldup(lcs->usereffective, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
 		case 'C': /* Forwarded server user name */
-			replace_val.replace_str = lcs->userforwarded;
+			replace_val.replace_str = strfilldup(lcs->userforwarded, "-");
+			enough_mem(replace_val.replace_str);
 			replace_type = LOG_REPLACE_STRING;
 			break;
+		case 'u': /* unix time, seconds since 1970 */
+			replace_val.replace_luint =
+					(unsigned long int) time(NULL);
+			replace_type = LOG_REPLACE_LUINT;
+			break;
+		case 'U': /* unix time, seconds since 1970 with milliseconds behind */
+			{
+				struct timeval tv;
+
+				replace_val.replace_str = malloc(100);
+				if (gettimeofday(&tv, NULL) < 0) {
+					tv.tv_sec = time(NULL);
+					tv.tv_usec = 0;
+				}
+				snprintf(replace_val.replace_str, 100,
+						"%lu.%03lu",
+						tv.tv_sec,
+						(tv.tv_usec / 1000));
+				replace_type = LOG_REPLACE_STRING;
+			}
+			break;
 		case '%': /* percent sign */
 			replace_val.replace_char = '%';
 			replace_type = LOG_REPLACE_CHAR;
@@ -322,8 +397,7 @@ char* log_replace_char(const char pattern, struct log_cmd_st* lcs) {
 
 		case LOG_REPLACE_STRING:
 			if (replace_val.replace_str) {
-				s = strdup(replace_val.replace_str);
-				enough_mem(s);
+				s = replace_val.replace_str;
 			} else {
 				s = strdup("");
 				enough_mem(s);
@@ -424,8 +498,6 @@ void log_cmd_ent(struct cmdlogent_t* lent, struct log_cmd_st* lcs) {
 
 	char* commandpattern;
 	size_t commandpatternsize;
-	static char time_string[LENGTH];
-	time_t nowtime;
 	char* ws;
 
 	commandpatternsize = strlen(lcs->cmd) + 3;
@@ -445,44 +517,24 @@ void log_cmd_ent(struct cmdlogent_t* lent, struct log_cmd_st* lcs) {
 
 	if (incommandpattern(lent->specs, commandpattern)) {
 		/* found, log it */
-		nowtime = time(NULL);
-
 		if (strcmp(lent->style, "commonlog") == 0) {
-			/* [18/Feb/2001:11:42:46 CET] */
-			strftime(time_string, LENGTH,
-				"[%d/%b/%Y:%H:%M:%S %Z]",
-				localtime(&nowtime));
-			if (checkbegin(lcs->cmd, "PASS")) {
-				fprintf(lent->logf, "\"PASS *\"");
-			} else {
-				fprintf(lent->logf, "\"%s\"", lcs->cmd);
-			}
-			fprintf(lent->logf, " %d %ld\n",
-					lcs->respcode, lcs->transferred);
-		} else if (strcmp(lent->style, "xferlog") == 0) {
-			char* replaced;
 			const char* line_pattern =
-				"%d %h %t %f %y %w _ %n %l %s 0 * %c";
-
-			/* Wed Feb 14 01:41:28 2001 */
-			strftime(time_string, LENGTH,
-				"%a %b %d %H:%M:%S %Y",
-				localtime(&nowtime));
-			replaced = log_replace_line(line_pattern, lcs);
-			fprintf(lent->logf, "%s %s\n", time_string, replaced);
-			lcs->filename = (char*) 0;
+				"%A %n %l %D \"%m\" %s %b";
+			char* replaced = log_replace_line(line_pattern, lcs);
+			fprintf(lent->logf, "%s\n", replaced);
+			free(replaced);
+		} else if (strcmp(lent->style, "xferlog") == 0) {
+			const char* line_pattern =
+				"%t %T %d %b \"%f\" %y _ %w %o %n %e 0 * %c";
+			char* replaced = log_replace_line(line_pattern, lcs);
+			fprintf(lent->logf, "%s\n", replaced);
 			free(replaced);
 		} else {
 			char* line_pattern = lent->style;
 			char* replaced;
 
-			/* Wed Feb 14 01:41:28 2001 */
-			strftime(time_string, LENGTH,
-				"%a %b %d %H:%M:%S %Y",
-				localtime(&nowtime));
 			replaced = log_replace_line(line_pattern, lcs);
-			fprintf(lent->logf, "%s %s\n", time_string, replaced);
-			lcs->filename = (char*) 0;
+			fprintf(lent->logf, "%s\n", replaced);
 			free(replaced);
 		}
 		fflush(lent->logf);
@@ -509,6 +561,7 @@ void log_cmd(struct log_cmd_st* lcs) {
 		log_cmd_ent(dirs, lcs);
 		dirs = dirs->next;
 	}
+	lcs->filename = (char*) 0;
 }
 
 static
@@ -528,14 +581,14 @@ int log_init_debuglevel() {
 	const char* dbl;
 
 	if (!(dbl = config_get_option("debuglevel"))) {
-		log(4, "No debug level specified. Using level 7");
+		jlog(4, "No debug level specified. Using level 7");
 		debuglevel = 7;
 	} else {
 		errno = 0;
 		/* with conv_char2* we wouldn't be able to log a warning */
 		debuglevel = strtol(dbl, (char**) 0, 10);
-		if (errno || debuglevel <= 0 || debuglevel > 9) {
-			log(4, "Invalid debuglevel specified: \"%s\". Using "
+		if (errno || debuglevel < 0 || debuglevel > 9) {
+			jlog(4, "Invalid debuglevel specified: \"%s\". Using "
 				"level 7", dbl);
 			debuglevel = 7;
 		}
@@ -683,7 +736,7 @@ int log_init_logfile(struct loginfo_st* cfg) {
 	if (!(cfg->logf = open_logfile(cfg->logf_name))) {
 		return -1;
 	}
-	log(7, "jftpgw v"JFTPGW_VERSION" opened the logfile");
+	jlog(7, "jftpgw v"JFTPGW_VERSION" opened the logfile");
 	return 0;
 }
 
@@ -705,13 +758,13 @@ int log_init_cmdlog(struct loginfo_st* cfg, int open) {
 	int count_specs = slist_count(opt_specs);
 
 	if (!(count_names == count_styles && count_styles == count_specs)) {
-		log(4, "Counted %d times \"cmdlogfile\" as option",
+		jlog(4, "Counted %d times \"cmdlogfile\" as option",
 				count_names);
-		log(4, "Counted %d times \"cmdlogfile-style\" as option",
+		jlog(4, "Counted %d times \"cmdlogfile-style\" as option",
 				count_styles);
-		log(4, "Counted %d times \"cmdlogfile-specs\" as option",
+		jlog(4, "Counted %d times \"cmdlogfile-specs\" as option",
 				count_specs);
-		log(4, "This is not balanced, please fix your configuration");
+		jlog(4, "This is not balanced, please fix your configuration");
 		return -1;
 	}
 
@@ -828,17 +881,17 @@ int log_init_dirlog(struct loginfo_st* cfg, int open) {
 			&& count_specs == count_prefix
 			&& count_prefix == count_suffix)) {
 
-		log(4, "Counted %d times \"connectionlogdir\" as option",
+		jlog(4, "Counted %d times \"connectionlogdir\" as option",
 				count_names);
-		log(4, "Counted %d times \"connectionlogdir-style\" as option",
+		jlog(4, "Counted %d times \"connectionlogdir-style\" as option",
 				count_styles);
-		log(4, "Counted %d times \"connectionlogdir-specs\" as option",
+		jlog(4, "Counted %d times \"connectionlogdir-specs\" as option",
 				count_specs);
-		log(4, "Counted %d times \"connectionlogdir-fileprefix\""
+		jlog(4, "Counted %d times \"connectionlogdir-fileprefix\""
 				" as option", count_prefix);
-		log(4, "Counted %d times \"connectionlogdir-filesuffix\""
+		jlog(4, "Counted %d times \"connectionlogdir-filesuffix\""
 				" as option", count_suffix);
-		log(4, "This is not balanced, please fix your configuration");
+		jlog(4, "This is not balanced, please fix your configuration");
 		return -1;
 	}
 
