@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -50,6 +50,7 @@ int handle_login(struct clientinfo* clntinfo) {
 	char *buffer = 0;
 	int ss, cs;
 	int i, expected;
+	int protoviolations = 0;
 
 	conn_info.lcs = &lcs;
 	conn_info.clntinfo = clntinfo;
@@ -62,7 +63,7 @@ int handle_login(struct clientinfo* clntinfo) {
 	    config_compare_option("logintime", "connect")) {
 
 		if (config_get_ioption("loginstyle", 0) != 0) {
-			log(5, "A login at the connection time only works with loginstyle == 0, setting loginstyle = 0");
+			jlog(5, "A login at the connection time only works with loginstyle == 0, setting loginstyle = 0");
 
 			config_option_list_delete("loginstyle");
 			config_option_list_add("loginstyle", "0");
@@ -75,7 +76,7 @@ int handle_login(struct clientinfo* clntinfo) {
 	expected = QUITFUNC + 1;    /* skip reset and quit function */
 	while (1) {
 		if (timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+			jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 			return -1;
 		}
 		if (buffer) {
@@ -108,7 +109,7 @@ int handle_login(struct clientinfo* clntinfo) {
 
 		if (!buffer) {
 			if (timeout) {
-				log(2, "Timeout in %s line %d\n", __FILE__
+				jlog(2, "Timeout in %s line %d\n", __FILE__
 						,__LINE__);
 				err_time_readline(cs);
 			} else {
@@ -117,11 +118,11 @@ int handle_login(struct clientinfo* clntinfo) {
 			return -1;
 		}
 
-		log(8, "Expecting %s", cmdhandler[expected].cmd);
-		if (my_strcasestr(buffer, "PASS ") == (char*) 0) {
-			log(8, "Got: %s", buffer);
+		jlog(8, "Expecting %s", cmdhandler[expected].cmd);
+		if (my_strcasestr(buffer, "PASS") == (char*) 0) {
+			jlog(8, "Got: %s", buffer);
 		} else {
-			log(8, "Got the password");
+			jlog(8, "Got the password");
 		}
 
 		/* check for QUIT */
@@ -141,6 +142,7 @@ int handle_login(struct clientinfo* clntinfo) {
 			int ret = (cmdhandler[expected].func)
 						(buffer, &conn_info);
 			memset(buffer, 0, strlen(buffer));
+			protoviolations = 0;
 			switch (ret) {
 				case CMD_DONE:
 					/* we're done - logged in */
@@ -168,6 +170,15 @@ int handle_login(struct clientinfo* clntinfo) {
 			}
 			/* found and called proper function */
 		} else {
+			protoviolations++;
+			if (protoviolations >=
+			    config_get_ioption("loginprotocolviolations", 10)){
+				/* like ABORT */
+				sayf(clntinfo->clientsocket,
+					"500 Too many consequent protocol "
+					"violations - Closing connection\r\n");
+				return -1;
+			}
 			sayf(clntinfo->clientsocket,
 				"530 Login incorrect. Expected %scommand\r\n",
 				cmdhandler[expected].cmd);
@@ -191,7 +202,7 @@ int login(struct clientinfo* clntinfo, int stage) {
 
 	if (! clntinfo->user && stage >= LOGIN_ST_USER) {
 		say(clntinfo->clientsocket, "500 Error logging in\r\n");
-		log(4, "No user was set - Cannot proceed");
+		jlog(4, "No user was set - Cannot proceed");
 		return CMD_ERROR;
 	}
 
@@ -202,9 +213,9 @@ int login(struct clientinfo* clntinfo, int stage) {
 
 	if (! clntinfo->destination) {
 		say(clntinfo->clientsocket, "500 Error logging in\r\n");
-		log(4, "No destination was set - Cannot proceed");
+		jlog(4, "No destination was set - Cannot proceed");
 		if (config_compare_option("logintime", "connect")) {
-			log(4, "This may be because of your logintime --> connect setting");
+			jlog(4, "This may be because of your logintime --> connect setting");
 		}
 		return CMD_ERROR;
 	}
@@ -329,9 +340,9 @@ int login_mayconnect(struct clientinfo* clntinfo) {
 			/* it's not a valid IP and we could not look it up,
 			 * so we could not get valid information about the
 			 * destination host */
-			log(7, "Nonsense destination (no IP and could not look up hostname): %s",
+			jlog(7, "Nonsense destination (no IP and could not look up hostname): %s",
 					clntinfo->destination);
-			log(8, "Please check your nameserver configuration. This may also happen if your chroot-environment does not contain the necessary files which the libc needs for a lookup");
+			jlog(8, "Please check your nameserver configuration. This may also happen if your chroot-environment does not contain the necessary files which the libc needs for a lookup");
 			allowed = 0;
 		}
 	} else {
@@ -342,7 +353,7 @@ int login_mayconnect(struct clientinfo* clntinfo) {
 	if (server_ip != (unsigned long) UINT_MAX) {
 		/* we could get a valid IP, now we can evaluate if client
 		 * is allowed to connect to the destination host */
-		log(9, "Checking all tags");
+		jlog(9, "Checking all tags");
 		tags = TAG_ALL_NOT_FORWARDED;
 		if (clntinfo->before_forward.user) {
 			/* if we are treating a forward, add this value, it
@@ -409,7 +420,7 @@ int login_mayconnect(struct clientinfo* clntinfo) {
 
 	if (!allowed) {
 		say(clntinfo->clientsocket, "531 You are not allowed to connect to that host.\r\n");
-		log(8, "Not allowed to connect to %s", clntinfo->destination);
+		jlog(8, "Not allowed to connect to %s", clntinfo->destination);
 		lcs.respcode = 531;
 		return CMD_ERROR;
 	}
@@ -464,7 +475,7 @@ int login_setforward_user(struct clientinfo* clntinfo) {
 	if (config_compare_option("access", "allow") == 0) {
 		say(clntinfo->clientsocket, "531 You are not allowed to "
 			"connect to that host. Goodbye.\r\n");
-		log(5, "%s was not allowed to connect.",
+		jlog(5, "%s was not allowed to connect.",
 					conv_ip_to_char(clntinfo->client_ip));
 		return CMD_ERROR;
 	}
@@ -476,9 +487,7 @@ int login_setforward_user(struct clientinfo* clntinfo) {
 		/* no forward */
 		lcs.userlogin = strnulldup(clntinfo->user);
 		lcs.usereffective = strnulldup(clntinfo->user);
-		if (lcs.userforwarded) {
-			free(lcs.userforwarded);
-		}
+		if (lcs.userforwarded) { free(lcs.userforwarded); }
 		lcs.userforwarded = strdup("<no forward>");
 		enough_mem(lcs.userforwarded);
 		return CMD_HANDLED;
@@ -492,6 +501,7 @@ int login_setforward_user(struct clientinfo* clntinfo) {
 	 *    forward johnfred@fooserver.com   JiKe94  johnspass
 	 * </user>
 	 *    forward *@fooserver.com:2378
+	 *    forward %@fooserver.com:2378
 	 */
 
 	/* split the list into tokens */
@@ -499,6 +509,36 @@ int login_setforward_user(struct clientinfo* clntinfo) {
 		return CMD_HANDLED;
 	}
 
+	/* if we have a defaultforward setting, the variable user has to be
+	 * defined so that we can decide if we can use defaultforward at
+	 * all. If this is not the case, but there is a defaultforward
+	 * setting, skip this whole part. This can happen by setting
+	 * logintime to "connect" */
+	if (!clntinfo->user &&
+	    forward_list->value &&
+	    forward_list->value[0] == '%' &&
+	    forward_list->value[1] == '@') {
+		slist_destroy(forward_list);
+		return CMD_HANDLED;
+	}
+
+	/* if there is a defaultforward but we already have a destination
+	 * set, we can quit here as well */
+	if (clntinfo->destination &&
+	    forward_list->value &&
+	    forward_list->value[0] == '%' &&
+	    forward_list->value[1] == '@') {
+		/* no forward is being used */
+		lcs.userlogin = strnulldup(clntinfo->user);
+		lcs.usereffective = strnulldup(clntinfo->user);
+		if (lcs.userforwarded) { free(lcs.userforwarded); }
+		lcs.userforwarded = strdup("<no forward>");
+		enough_mem(lcs.userforwarded);
+		slist_destroy(forward_list);
+		return CMD_HANDLED;
+	}
+
+
 	forward_list_cur = forward_list;
 
 	/* read and save the values */
@@ -571,6 +611,24 @@ int login_setforward_user(struct clientinfo* clntinfo) {
 		clntinfo->forward.passauth = 1;
 	}
 
+	/* If there is still no destination, see if we have a defaultforward
+	 * setting */
+	if (clntinfo->destination == (char*) 0 &&
+	    clntinfo->user &&
+	    clntinfo->forward.login[0] == '%' &&
+	    clntinfo->forward.login[1] == '@') {
+		/* defaultforward */
+		int newsize = strlen(&(clntinfo->forward.login[1])) +
+				strlen(clntinfo->user) + 1;
+		char* tmp = (char*) malloc(newsize);
+		enough_mem(tmp);
+		snprintf(tmp, newsize, "%s%s", clntinfo->user,
+					&(clntinfo->forward.login[1]));
+		free(clntinfo->forward.login);
+		clntinfo->forward.login = tmp;
+		jlog(8, "No destination was set. Using %s because of defaultforward setting", clntinfo->forward.login);
+	}
+
 	/* call the parsing routine and let it set the values */
 	if (set_userdest(clntinfo->forward.login, 0, clntinfo, "@,: \t") < 0) {
 		return CMD_ERROR;
@@ -662,7 +720,7 @@ int login_init_connection(struct clientinfo* clntinfo) {
 				clntinfo->destination,
 				clntinfo->destinationport,
 				strerror(err));
-		log(5, ERR_STR_P2, clntinfo->destination,
+		jlog(5, ERR_STR_P2, clntinfo->destination,
 				   clntinfo->destinationport,
 				   strerror(err));
 		return CMD_ERROR;
@@ -695,26 +753,30 @@ int login_connected_setup(struct clientinfo* clntinfo) {
 		clntinfo->login.welcomemsg.lastmsg = (char*) 0;
 	}
 
-	say(clntinfo->serversocket, "SYST\r\n");
-	buffer = ftp_readline(clntinfo->serversocket);
-	if (!buffer) {
-		if (timeout) {
-			log(1, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
-			err_time_readline(clntinfo->clientsocket);
-			return CMD_ABORT;
-		} else {
-			err_readline(clntinfo->clientsocket);
-			return CMD_ABORT;
+	if (config_get_bool("initialsyst") == 1) {
+		say(clntinfo->serversocket, "SYST\r\n");
+		buffer = ftp_readline(clntinfo->serversocket);
+		if (!buffer) {
+			if (timeout) {
+				jlog(1, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+				err_time_readline(clntinfo->clientsocket);
+				return CMD_ABORT;
+			} else {
+				err_readline(clntinfo->clientsocket);
+				return CMD_ABORT;
+			}
+		}
+		if (!strlen(buffer)) {
+			jlog(2, "The server did not respond to the initial SYST command");
+			say(clntinfo->clientsocket,
+					"The server did not respond correctly\r\n");
+			free(buffer);
+			return CMD_ERROR;
 		}
-	}
-	if (!strlen(buffer)) {
-		log(2, "The server did not respond to the initial SYST command");
-		say(clntinfo->clientsocket,
-				"The server did not respond correctly\r\n");
 		free(buffer);
-		return CMD_ERROR;
+	} else {
+		jlog(9, "Suppressing the initial SYST command");
 	}
-	free(buffer);
 	buffer = (char*) 0;
 
 	return CMD_HANDLED;
@@ -728,7 +790,7 @@ int login_readwelcome(struct clientinfo *clntinfo) {
 	if (!clntinfo->login.welcomemsg.fullmsg) {
 		/* an error occurred */
 		if (timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+			jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 			err_time_readline(clntinfo->clientsocket);
 		} else {
 			set_errstr("Server closed the connection");
@@ -737,11 +799,11 @@ int login_readwelcome(struct clientinfo *clntinfo) {
 		return CMD_ABORT;
 	}
 
-	log(9, "Connected to %s, got \"%s\" as welcome message",
+	jlog(9, "Connected to %s, got \"%s\" as welcome message",
 		clntinfo->destination, clntinfo->login.welcomemsg.fullmsg);
 
 	if (!checkbegin(clntinfo->login.welcomemsg.lastmsg, "220 ")) {
-		log(2, "Not a valid FTP server response (%s)",
+		jlog(2, "Not a valid FTP server response (%s)",
 				clntinfo->login.welcomemsg.fullmsg);
 		say(clntinfo->clientsocket,clntinfo->login.welcomemsg.fullmsg);
 
@@ -772,7 +834,7 @@ int login_sendauth_user(struct clientinfo* clntinfo) {
 	free(sendbuf);
 	sendbuf = 0;
 	if (ret < 0) {
-		log(2, "Error writing the user name to the server: %s",
+		jlog(2, "Error writing the user name to the server: %s",
 			strerror(errno));
 		return CMD_ABORT;
 	}
@@ -781,7 +843,7 @@ int login_sendauth_user(struct clientinfo* clntinfo) {
 
 	if (!clntinfo->login.authresp.fullmsg) {
 		if (timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+			jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 			err_time_readline(clntinfo->clientsocket);
 		} else {
 			err_readline(clntinfo->clientsocket);
@@ -814,7 +876,7 @@ int login_sendauth_pass(struct clientinfo* clntinfo) {
 		say(clntinfo->clientsocket, sendbuf);
 		free(sendbuf);
 
-		log(7, "Got \"%s\" after sending the username.", buffer);
+		jlog(7, "Got \"%s\" after sending the username.", buffer);
 		free(clntinfo->login.authresp.fullmsg);
 		free(clntinfo->login.welcomemsg.fullmsg);
 		clntinfo->login.welcomemsg.fullmsg = (char*) 0;
@@ -843,7 +905,7 @@ int login_sendauth_pass(struct clientinfo* clntinfo) {
 			clntinfo->pass = (char*) 0;
 		} else {
 			memset(sendbuf, (char) 0, sendbufsize);
-			clntinfo->anon_user = strdup("UNKNOWN");
+			clntinfo->anon_user = (char*) 0;
 		}
 		free(sendbuf);
 		free(userdup);
@@ -853,7 +915,7 @@ int login_sendauth_pass(struct clientinfo* clntinfo) {
 		clntinfo->login.authresp.lastmsg   = (char*) 0;
 
 		if (ret < 0) {
-			log(2, "Error writing the password to the server: %s",
+			jlog(2, "Error writing the password to the server: %s",
 				strerror(errno));
 			free(clntinfo->login.welcomemsg.fullmsg);
 			clntinfo->login.welcomemsg.fullmsg = (char*) 0;
@@ -869,7 +931,7 @@ static
 int login_loggedin_setup(struct clientinfo* clntinfo) {
 
 	/* we seem to have a successful login */
-	log(7, "Logged in to %s as %s!",
+	jlog(7, "Logged in to %s as %s!",
 		clntinfo->destination, clntinfo->user);
 
 	/* initialize the log_cmd_st structure with the values that have to
@@ -877,8 +939,6 @@ int login_loggedin_setup(struct clientinfo* clntinfo) {
 	lcs.anon_user = clntinfo->anon_user;
 	lcs.svrip = strdup(get_char_ip(GET_IP_SERVER, clntinfo));
 	lcs.svrname = hostent_get_name(&hostcache, inet_addr(lcs.svrip));
-	lcs.ifipsvr = strdup(conv_ip_to_char(clntinfo->addr_to_server));
-	enough_mem(lcs.ifipsvr);
 	lcs.svrlogin = clntinfo->destination;
 
 	/* we don't need the configuration sections and the backup anymore */
@@ -891,17 +951,19 @@ int login_loggedin_setup(struct clientinfo* clntinfo) {
 
 	clntinfo->addr_to_server =
 			socketinfo_get_local_ip(clntinfo->serversocket);
+	lcs.ifipsvr = strdup(conv_ip_to_char(clntinfo->addr_to_server));
+	enough_mem(lcs.ifipsvr);
 
 	clntinfo->data_addr_to_client
 				= config_get_addroption("dataclientaddress",
 						clntinfo->addr_to_client);
-	log(8, "Got %s as data client address",
+	jlog(8, "Got %s as data client address",
 	       inet_ntoa(*((struct in_addr*) &clntinfo->data_addr_to_client)));
 
 	clntinfo->data_addr_to_server =
 				config_get_addroption("dataserveraddress",
 						clntinfo->addr_to_server);
-	log(8, "Got %s as data server address",
+	jlog(8, "Got %s as data server address",
 	       inet_ntoa(*((struct in_addr*) &clntinfo->data_addr_to_server)));
 
 	if (clntinfo->servermode == UNSPEC) {
@@ -926,7 +988,7 @@ int login_finish_login(struct clientinfo* clntinfo) {
 	clntinfo->login.authresp = readall(clntinfo->serversocket);
 	if (!clntinfo->login.authresp.fullmsg) {
 		if (timeout) {
-			log(2, "Timeout in %s line %d\n", __FILE__,
+			jlog(2, "Timeout in %s line %d\n", __FILE__,
 					__LINE__);
 			err_time_readline(clntinfo->clientsocket);
 		}
@@ -959,7 +1021,7 @@ int login_finish_login(struct clientinfo* clntinfo) {
 	if (!checkdigits(clntinfo->login.authresp.lastmsg, 230)) {
 		say(clntinfo->clientsocket,
 				clntinfo->login.authresp.lastmsg);
-		log(8, "Got \"%s\" after sending the password",
+		jlog(8, "Got \"%s\" after sending the password",
 				clntinfo->login.authresp.fullmsg);
 		free(clntinfo->login.authresp.fullmsg);
 		free(buffer);
@@ -1015,7 +1077,7 @@ int login_failed(struct clientinfo* clntinfo) {
 	 * the backup is used */
 
 	if (config_activate_backup() < 0) {
-		log(6, "Backup could not be activated - can't continue with login procedure");
+		jlog(6, "Backup could not be activated - can't continue with login procedure");
 		return CMD_ABORT;
 	}
 
