@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,7 +27,13 @@
 #include <linux/netfilter_ipv4.h>
 #endif
 #ifdef HAVE_NETINET_IP_FIL_H
+/* there are reports that NetBSD 2.0E does not have IPL_NAT so that it would
+ * not compile */
+#ifdef IPL_NAT
 #include "support/ipfilter.c"
+#else
+#warning ip_fil.h found but no IPL_NAT available...
+#endif
 #endif
 
 extern struct hostent_list* hostcache;
@@ -46,7 +52,7 @@ int getanylocalport(int sd, struct sockaddr_in* sin, int needroot) {
 	if (ret < 0 && errno == EADDRNOTAVAIL
 			&& sin->sin_addr.s_addr != INADDR_ANY) {
 		sin->sin_addr.s_addr = INADDR_ANY;
-		log(6, "Couldn't assign requested address, trying any address");
+		jlog(6, "Couldn't assign requested address, trying any address");
 		ret = bind(sd, (struct sockaddr*) sin,
 						sizeof(struct sockaddr));
 	}
@@ -55,7 +61,7 @@ int getanylocalport(int sd, struct sockaddr_in* sin, int needroot) {
 		return -1;
 	}
 	if (ret < 0) {
-		log(2, "Could not bind to a free port: %s",
+		jlog(2, "Could not bind to a free port: %s",
 				strerror(errno));
 		return -1;
 	} else {
@@ -102,7 +108,7 @@ int getportinrange(int sd, struct sockaddr_in* sin,
 	do {
 		unsigned int inner_index;
 		if (!prscur) {
-			log(2, "prscur was NIL in %s, %d", __FILE__, __LINE__);
+			jlog(2, "prscur was NIL in %s, %d", __FILE__, __LINE__);
 			break;
 		}
 		inner_index = prscur->endport + 1;
@@ -170,14 +176,14 @@ int getportinrange(int sd, struct sockaddr_in* sin,
 				return -1;
 			}
 			if (ret < 0) {
-				log(9, "Tried port %s:%d in vain: %s",
+				jlog(9, "Tried port %s:%d in vain: %s",
 						inet_ntoa(sin->sin_addr),
 						pasv_ports[pasv_port_index],
 						strerror(errno));
 				tries++;
 			} else {
 				found_pasv_port = 1;
-				log(8, "Found free port %d after %d tries",
+				jlog(8, "Found free port %d after %d tries",
 						pasv_ports[pasv_port_index],
 						tries);
 			}
@@ -194,7 +200,7 @@ int getportinrange(int sd, struct sockaddr_in* sin,
 		 * configuration.  -- TJ
 		 */
 
-		log(4, "unable to find a free port in the port range"
+		jlog(4, "unable to find a free port in the port range"
 		"; defaulting to INPORT_ANY");
 		return getanylocalport(sd, sin, 1);
 	}
@@ -220,13 +226,13 @@ int openport(struct sockaddr_in sin,
 		return -1;
 	}
 	if ((handle = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
-		log(2, "Could not create socket in openport(): %s",
+		jlog(2, "Could not create socket in openport(): %s",
 							strerror(errno));
 		return -1;
 	}
 	if (setsockopt(handle, SOL_SOCKET, SO_REUSEADDR,
 					(void*)&one, sizeof(one)) < 0) {
-		log(3, "Could not set SO_REUSEADDR on socket in openport()"
+		jlog(3, "Could not set SO_REUSEADDR on socket in openport()"
 				": %s", strerror(errno));
 		/* do not return */
 	}
@@ -235,7 +241,7 @@ int openport(struct sockaddr_in sin,
 		return -1;
 	}
 	if (handle < 0) {
-		log(2, "Error opening the socket: %s", strerror(errno));
+		jlog(2, "Error opening the socket: %s", strerror(errno));
 		return -1;
 	}
 
@@ -243,13 +249,16 @@ int openport(struct sockaddr_in sin,
 	dp.sin_family = AF_INET;
 	dp.sin_addr.s_addr = local_address;
 
+	jlog(9, "Trying to get a free source port on address %s",
+						inet_ntoa(dp.sin_addr));
+
 	/* will default to getanylocalport if localportrange is not defined
 	 * */
 	handle = getportinrange(handle, &dp, localportrange);
 
 	if (connect(handle, (struct sockaddr*) &sin, sizeof(sin)) < 0) {
 		int err = errno;
-		log(1, "Error connecting to %s:%d: %s",
+		jlog(1, "Error connecting to %s:%d: %s",
 						inet_ntoa(sin.sin_addr),
 						ntohs(sin.sin_port),
 						strerror(err));
@@ -281,18 +290,20 @@ int openportname(const char* hostname,
 
 	unsigned long int host_ip;
 	struct sockaddr_in sin;
+	struct in_addr local_addr;
+	local_addr.s_addr = local_address;
 
 	if (local_address != INADDR_ANY) {
-		log(8, "Using special address %s to connect to %s "
+		jlog(8, "Using special address %s to connect to %s "
 				"on port %d",
-			inet_ntoa(*((struct in_addr*)&local_address)),
+			inet_ntoa(local_addr),
 			hostname, port);
 	}
 
 	memset((void*)&sin, 0, sizeof(sin));
 	host_ip = hostent_get_ip(&hostcache, hostname);
 	if (host_ip == (unsigned long int) UINT_MAX) {
-		log(3, "Could not look up %s", hostname);
+		jlog(3, "Could not look up %s", hostname);
 		return -1;
 	}
 	sin.sin_family         = AF_INET;
@@ -316,13 +327,13 @@ int openlocalport(struct sockaddr_in *sin,           /* return */
 	int one = 1;
 
 	if ((sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
-		log(1, "Could not create socket to bind to a free port: %s",
+		jlog(1, "Could not create socket to bind to a free port: %s",
 				strerror(errno));
 		return -1;
 	}
 	if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR,
 				(void*)&one, sizeof(one)) < 0) {
-		log(3, "Could not set SO_REUSEADDR in open_localport: %s",
+		jlog(3, "Could not set SO_REUSEADDR in open_localport: %s",
 							strerror(errno));
 		/* do not return */
 	}
@@ -332,11 +343,11 @@ int openlocalport(struct sockaddr_in *sin,           /* return */
 	sin->sin_family = AF_INET;
 
 	if(getportinrange(sd, sin, prs) < 0) {
-		log(2, "Could not bind to a local port: %s", strerror(errno));
+		jlog(2, "Could not bind to a local port: %s", strerror(errno));
 		return -1;
 	}
 	if (listen(sd, 1) < 0) {
-		log(2, "Could not listen on a free port: %s", strerror(errno));
+		jlog(2, "Could not listen on a free port: %s", strerror(errno));
 		return -1;
 	}
 
@@ -344,7 +355,7 @@ int openlocalport(struct sockaddr_in *sin,           /* return */
 
 	slen = sizeof(struct sockaddr);
 	if (getsockname(sd, (struct sockaddr*) sin, &slen) < 0) {
-		log(2, "getsockname failed after binding to a free port: %s",
+		jlog(2, "getsockname failed after binding to a free port: %s",
 				strerror(errno));
 		return -1;
 	}
@@ -405,24 +416,32 @@ struct sockaddr_in get_showaddr(int shandle, int type) {
 	slen = sizeof (sin);
 	if (type == SHOW_ORIGINAL_IP) {
 #ifdef HAVE_NETINET_IP_FIL_H
+/* if IPL_NAT is not defined, we have not included the file with the
+ * ipfilter_get_real_dst() function */
+#ifdef IPL_NAT
 		if (changeid(PRIV, UID, "ipfilter") < 0) {
-			log(2, "failed to gain privileges (ipnat device)");
+			jlog(2, "failed to gain privileges (ipnat device)");
 			i = -1;
 		} else {
 			i = ipfilter_get_real_dst(shandle, &sin) ? 0 : -1;
 		}
 		if (changeid(UNPRIV, EUID, "ipfilter") < 0) {
-			log(2, "fail to drop privileges (ipnat device)");
+			jlog(2, "fail to drop privileges (ipnat device)");
 			i = -1;
 		}
 #else
+/* IPL_NAT not defined but HAVE_NETINET_IP_FIL_H defined */
+		i = nf_getsockname(shandle, (struct sockaddr*) &sin, &slen);
+#endif
+#else
+/* HAVE_NETINET_IP_FIL_H not defined */
 		i = nf_getsockname(shandle, (struct sockaddr*) &sin, &slen);
 #endif
 	} else {
 		i = getsockname(shandle, (struct sockaddr*) &sin, &slen);
 	}
 	if (i != 0) {
-		log(2, "getsockname failed. Can't get the IP of the interface");
+		jlog(2, "getsockname failed. Can't get the IP of the interface");
 		sin.sin_addr.s_addr = -1;
 		sin.sin_port = 0;
 		return sin;
@@ -478,17 +497,22 @@ unsigned long int get_local_addr_by_sending_icmp(unsigned long int to_addr) {
 	struct sockaddr_in sin;
 	int i, s, len;
 	unsigned int ident = getpid() & 0xFFFF;
+#ifdef HAVE_SOCKLEN_T
+	socklen_t size;
+#else
+	int size;
+#endif
 
 	memset(&sin, 0, sizeof(struct sockaddr_in));
 	sin.sin_port = 0;
 	sin.sin_addr.s_addr = to_addr;
 
 	if (!(proto = getprotobyname("icmp"))) {
-		log(4, "protocol icmp unknown");
+		jlog(4, "protocol icmp unknown");
 		return ULONG_MAX;
 	}
 	if ((s = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
-		log(4, "Error creating the socket: %s", strerror(errno));
+		jlog(4, "Error creating the socket: %s", strerror(errno));
 		return ULONG_MAX;
 	}
 
@@ -504,7 +528,7 @@ unsigned long int get_local_addr_by_sending_icmp(unsigned long int to_addr) {
 	i = sendto(s, (char *)icmp_packet, ICMP_SIZE,
 			0, (struct sockaddr*) &sin, sizeof(struct sockaddr));
 	if (i < 0) {
-		log(4, "Error in sendto: %s", strerror(errno));
+		jlog(4, "Error in sendto: %s", strerror(errno));
 		close(s);
 	}
 
@@ -513,19 +537,19 @@ unsigned long int get_local_addr_by_sending_icmp(unsigned long int to_addr) {
 
 	/* Now receive the packet again */
 	do {
-		i = sizeof(struct sockaddr);
+		size = sizeof(struct sockaddr);
 		len = recvfrom(s, (char*) ip_packet, IP_SIZE,  0,
-						(struct sockaddr*) &sin, &i);
+					(struct sockaddr*) &sin, &size);
 		if (len < 0) {
 			if (errno == EINTR) {
 				/* the answer did not come back after 5
 				 * seconds */
-				log(4, "the ICMP Echo Reply did not come "
+				jlog(4, "the ICMP Echo Reply did not come "
 						"back within 5 seconds");
 				close(s);
 				return ULONG_MAX;
 			}
-			log(4, "Error in recvfrom: %s", strerror(errno));
+			jlog(4, "Error in recvfrom: %s", strerror(errno));
 		}
 
 		/* multiply header length by 32 bits, i.e. 4 Bytes
@@ -549,25 +573,25 @@ unsigned long int get_local_addr_by_sending_udp(unsigned long int to_addr,
 	struct sockaddr_in sin;
 
 	if ((sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-		log(3, "Cannot create socket to send UDP packet to the transparent proxy client");
+		jlog(3, "Cannot create socket to send UDP packet to the transparent proxy client");
 		return ULONG_MAX;
 	}
 	sin.sin_family = AF_INET;
 	sin.sin_port = htons(port);
 	sin.sin_addr.s_addr = to_addr;
 	if (connect(sd, (struct sockaddr*) &sin, sizeof(sin)) < 0) {
-		log(5, "Cannot connect to the transparent proxy client");
+		jlog(5, "Cannot connect to the transparent proxy client");
 		close(sd);
 		return ULONG_MAX;
 	}
 	if (send(sd, "jftpgw", 6, 0) < 0) {
-		log(5, "Cannot send UDP packet to the transparent proxy client");
+		jlog(5, "Cannot send UDP packet to the transparent proxy client");
 		close(sd);
 		return ULONG_MAX;
 	}
 	i = sizeof(sin);
 	if (getsockname(sd, (struct sockaddr*) &sin, &i) < 0) {
-		log(2, "getsockname() failed to determine our IP (transparent proxy -> udp)");
+		jlog(2, "getsockname() failed to determine our IP (transparent proxy -> udp)");
 		close(sd);
 		return ULONG_MAX;
 	}
@@ -585,18 +609,18 @@ unsigned long int get_local_addr_by_sending(unsigned long int to_addr) {
 	if (strcasecmp(opt, "icmp") == 0) {
 		if (geteuid() != 0 && getuid() == 0) {
 			if (changeid(PRIV, UID, "Sending ICMP") < 0) {
-				log(4, "Trying a UDP packet");
+				jlog(4, "Trying a UDP packet");
 				goto try_udp;
 			} else {
 				changedid = 1;
 			}
 		} else if (getuid() != 0) {
-			log(4, "ICMP can only be sent by root, trying a UDP packet");
+			jlog(4, "ICMP can only be sent by root, trying a UDP packet");
 			goto try_udp;
 		}
 		addr = get_local_addr_by_sending_icmp(to_addr);
 		if (addr == ULONG_MAX) {
-			log(4, "Sending an ICMP packet failed, "
+			jlog(4, "Sending an ICMP packet failed, "
 						"trying a UDP packet");
 			goto try_udp;
 		}
@@ -605,7 +629,7 @@ unsigned long int get_local_addr_by_sending(unsigned long int to_addr) {
 				return ULONG_MAX;
 			}
 		}
-		log(9, "Getting IP by ICMP successful");
+		jlog(9, "Getting IP by ICMP successful");
 		return addr;
 	}
 try_udp:
@@ -626,14 +650,14 @@ unsigned long int socketinfo_get_local_addr_by_sending(int fd) {
 
 	/* Determine the IP the client sees of us */
 	if (getpeername(fd, (struct sockaddr *) &sin, &namelen) != 0) {
-		log(3, "Could not get peername for TP: %s", strerror(errno));
+		jlog(3, "Could not get peername for TP: %s", strerror(errno));
 	}
 	ret = get_local_addr_by_sending(sin.sin_addr.s_addr);
 
 
 	/* debug */
 	sin.sin_addr.s_addr = ret;
-	log(8, "Our IP to the client is %s", inet_ntoa(sin.sin_addr));
+	jlog(8, "Our IP to the client is %s", inet_ntoa(sin.sin_addr));
 
 	return ret;
 }
