@@ -50,7 +50,9 @@
 
 /* Define this as a fall through, HAVE_STDARG_H is probably already set */
 
-#define HAVE_VARARGS_H
+#ifndef HAVE_VARARGS_H
+#  define HAVE_VARARGS_H
+#endif
 
 /* varargs declarations: */
 
@@ -116,8 +118,8 @@ static void dopr_outch (char *buffer, size_t *currlen, size_t maxlen, char c );
 #define DP_C_LDOUBLE 3
 
 #define char_to_int(p) (p - '0')
-#ifndef MAX
-#define MAX(p,q) ((p >= q) ? p : q)
+#ifndef MAX_VAL
+#define MAX_VAL(p,q) ((p >= q) ? p : q)
 #endif
 
 static void dopr (char *buffer, size_t maxlen, const char *format, va_list args)
@@ -454,12 +456,12 @@ static void fmtint (char *buffer, size_t *currlen, size_t maxlen,
   convert[place] = 0;
 
   zpadlen = max - place;
-  spadlen = min - MAX (max, place) - (signvalue ? 1 : 0);
+  spadlen = min - MAX_VAL (max, place) - (signvalue ? 1 : 0);
   if (zpadlen < 0) zpadlen = 0;
   if (spadlen < 0) spadlen = 0;
   if (flags & DP_F_ZERO)
   {
-    zpadlen = MAX(zpadlen, spadlen);
+    zpadlen = MAX_VAL(zpadlen, spadlen);
     spadlen = 0;
   }
   if (flags & DP_F_MINUS) 
