@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -45,7 +45,7 @@ char* errstr = 0;
 
 void enough_mem(const void* ptr) {
 	if (ptr == (void*) 0) {
-		log(0, "Not enough memory for malloc. Exiting.");
+		jlog(0, "Not enough memory for malloc. Exiting.");
 		exit(1);
 	}
 }
@@ -136,28 +136,28 @@ int say(int fd, const char* phrase) {
 
 	i = select(fd + 1, NULL, &writeset, NULL, &writetime);
 	if (i == 0) {
-		log(2, "Timeout reached in say()");
-		log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
+		jlog(2, "Timeout reached in say()");
+		jlog(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 		timeout = 1;
 		return -1;
 	}
 	if (i == -1) {
-		log(1, "Error in select() in say(): %s", strerror(errno));
+		jlog(1, "Error in select() in say(): %s", strerror(errno));
 		return -1;
 	}
 
 	if (my_strcasestr(phrase, "PASS ") == (char*) 0) {
-		log(9, "Write(%d): %s", fd, phrase);
+		jlog(9, "Write(%d): %s", fd, phrase);
 	} else {
-		log(9, "Write(%d): ***hidden***", fd);
+		jlog(9, "Write(%d): ***hidden***", fd);
 	}
 	i = write(fd, phrase, strlen(phrase));
 	if (i < 0) {
-		log(3, "write (say) failed: %s", strerror(errno));
+		jlog(3, "write (say) failed: %s", strerror(errno));
 		if (my_strcasestr(phrase, "PASS ") == (char*) 0) {
-			log(3, "should say %s to %d", phrase, fd);
+			jlog(3, "should say %s to %d", phrase, fd);
 		} else {
-			log(3, "should say ***hidden*** to %d", fd);
+			jlog(3, "should say ***hidden*** to %d", fd);
 		}
 	}
 	return i;
@@ -178,110 +178,125 @@ int sayf(int fd, const char* fmt, ...) {
 int changeid(int who_id, int what, const char* reason) {
 	int i = 0;
 	char* wstr;
-	const char* who = (char*) 0;
+	const char* user = (char*) 0;
+	const char* group = (char*) 0;
+	const char* s_user = (char*) 0;
+	const char* s_group = (char*) 0;
 	uid_t uid  = getuid();
 	uid_t euid = geteuid();
-	gid_t gid  = getgid();
+
+	uid_t tuid; /* Target ids we want to change to */
+	uid_t teuid;
+	gid_t tgid;
+	gid_t tegid;
+
+	uid_t s_uid;
+	gid_t s_gid;
+
+	uid_t value; /* just for the logging statement */
 
 	if (who_id == PRIV) {
-		who = "root";
-	} else {
-		if (what == EUID || what == UID) {
-			who = config_get_option("runasuser");
-			if (!who) {
-				if (what == EUID) {
-					return 0;
-				}
-				if (what == UID) {
-					return 0;
-				}
-			}
-		}
-		if (what == EGID || what == GID) {
-			who = config_get_option("runasgroup");
-			if (!who) {
-				if (what == EGID) {
-					log(4, "No runasgroup option specified (but only egid)!");
-					return 0;
-				}
-				if (what == GID) {
-					log(4, "No runasgroup option specified!");
-					return -1;
-				}
-			}
-		}
-	}
-
-	/* we can't change the UID/GID if we are not root and if we don't
-	 * want to become root */
-	if (strcmp(who, "root") != 0  &&  uid != 0) {
-		return 0;
-	}
-	if (strcmp(who, "root") == 0  &&  euid == 0 
-			&& (what == EUID || what == UID)) {
-		/* we are done - root already */
-		return 0;
-	}
-	if (uid != 0) {
+		tuid = 0; value = 0;
+		wstr = "UID";
 		/* can't change the (E)(U/G)ID if we are not root */
-		return 0;
-	}
-	if (uid == 0 && strcmp(who, "root") != 0 && euid != 0
-			&& (what == UID || what == GID || what == EGID)) {
-		/* UID is root, but effective UID is not. The UID/GID should
-		 * be changed. Change to root first */
-		if (changeid(PRIV, UID, "Want to change UID/GID. "
-					"Change back to root first") < 0) {
-			return -1;
+		if (uid != 0) {
+			return 0;
+		}
+		/* we are done if we are root already */
+		if (euid == 0) {
+			return 0;
+		}
+		i = setuid(tuid);
+	} else {
+		/* who_id == UNPRIV, s_* = "source_*" */
+		s_user = runasuser.username;
+		s_uid = runasuser.uid;
+		s_group = runasuser.groupname;
+		s_gid = runasuser.gid;
+		user = config_get_option("runasuser");
+		group = config_get_option("runasgroup");
+
+		if (what == UID || what == EUID) {
+			/* no user is set - ok */
+			if (!user) {
+				return 0;
+			}
+			/* down here we have a user set */
+			if (!s_user || strcmp(user, s_user) != 0) {
+				jlog(2, "Invalid user specified: %s", user);
+			}
+		}
+		if (what == GID || what == EGID) {
+			/* no group is set - ok */
+			if (!group) {
+				return 0;
+			}
+			/* down here we have a group set */
+			if (!s_group || strcmp(group, s_group) != 0) {
+				jlog(2, "Invalid user specified: %s", group);
+			}
+		}
+		/* can't change the (E)(U/G)ID if we are not root */
+		if (uid != 0) {
+			return 0;
+		}
+		if (euid != 0) {
+			/* UID is root, but effective UID is not. The GID
+			 * should be changed. Change to root first. */
+			if (changeid(PRIV, UID, "Want to change UID/GID. "
+				"Changing back to root first") < 0) {
+				return -1;
+			}
+			/* Make sure that each call to changeid(..., GID, ...)
+			 * is followed by one to changeid(..., UID, ...) */
+		}
+		jlog(7, "%s", reason);
+		switch(what) {
+			case EUID:
+				wstr = "EUID";
+				teuid = s_uid; value = teuid;
+#ifdef HAVE_SETEUID
+				i = seteuid(teuid);
+#else
+				/* HP-UX does not know seteuid() */
+				i = setreuid(-1, euid);
+#endif
+				break;
+			case UID:
+				wstr = "UID";
+				tuid = s_uid; value = tuid;
+				/* we are done if we are root already */
+				if (tuid == 0 && euid == 0) {
+					return 0;
+				}
+				i = setuid(tuid);
+				break;
+			case EGID:
+				wstr = "EGID";
+				tegid = s_gid; value = tegid;
+#ifdef HAVE_SETEGID
+				i = setegid(tegid);
+#else
+				/* HP-UX does not know setegid() */
+				i = setregid(-1, tegid);
+#endif
+				break;
+			case GID:
+				wstr = "GID";
+				tgid = s_gid; value = tgid;
+				i = setgid(tgid);
+				break;
+			default:
+				wstr = "ERROR";
+				value = 0;
 		}
 	}
 
-	log(7, reason);
-
-	if (strcmp(who, "root") == 0) {
-		uid = 0;
-	} else if (strcmp(who, runasuser.username) == 0) {
-		uid = runasuser.uid;
-	} else if (strcmp(who, runasuser.groupname) == 0) {
-		gid = runasuser.gid;
-	} else {
-		log(2, "Invalid user/group specified: %s", who);
-		return -1;
-	}
-	if (what == EUID) {
-#ifdef HAVE_SETEUID
-		i = seteuid(uid);
-#else
-		/* HP-UX does not know seteuid() */
-		i = setreuid(-1, uid);
-#endif
-	} else if (what == UID) {
-		i = setuid(uid);
-	} else if (what == EGID) {
-#ifdef HAVE_SETEGID
-		i = setegid(gid);
-#else
-		/* HP-UX does not know setegid() */
-		i = setregid(-1, gid);
-#endif
-	} else  if (what == GID) {
-		i = setgid(gid);
-	}
-	switch (what) {
-		case UID: wstr = "UID"; break;
-		case EUID: wstr = "EUID"; break;
-		/* map gid on uid */
-		case GID: wstr = "GID"; uid = gid; break;
-		/* map gid on uid */
-		case EGID: wstr = "EGID"; uid = gid; break;
-		default: wstr = "ERROR";
-	}
 	if (i) {
-		log(3, "Could not change the %s to %d: %s",
-				wstr, uid, strerror(errno));
-	}
-	else {
-		log(8, "Changed %s to %d (%s)", wstr, uid, who);
+		jlog(3, "Could not change the %s to %d: %s",
+				wstr, value, strerror(errno));
+	} else {
+		jlog(8, "Changed %s to %d", wstr, value);
 	}
 	return i;
 }
@@ -292,7 +307,11 @@ void reap_chld_info (int signo) {
 	int err = errno;
 	int status;
 	/* signal handler but waitpid() is reentrant */
-	while (waitpid (-1, &status, WNOHANG) > 0) {};
+	while (waitpid (-1, &status, WNOHANG) > 0
+		|| errno == EINTR) {
+
+		errno = 0;
+	};
 	errno = err;
 }
 
@@ -309,14 +328,17 @@ int get_chld_pid() {
 	int status;
 	pid_t pid;
 
-	while ((pid = waitpid (-1, &status, WNOHANG)) > 0) {
-		log(9, "A child exited. Pid: %d", pid);
-		log(9, "unregistering pid ...");
+	while ((pid = waitpid (-1, &status, WNOHANG)) > 0
+		|| errno == EINTR) {
+
+		errno = 0;
+		jlog(9, "A child exited. Pid: %d", pid);
+		jlog(9, "unregistering pid ...");
 		ret = unregister_pid(pid);
 		if (ret) {
-			log(3, "Error unregistering pid");
+			jlog(3, "Error unregistering pid");
 		} else {
-			log(9, "unregistered");
+			jlog(9, "unregistered");
 		}
 	}
 	chlds_exited = 0;
@@ -324,18 +346,6 @@ int get_chld_pid() {
 }
 
 
-/* returns the first characters up to a whitespace character */
-
-/* Checks if PATTERN is the beginning of RESPONSE */
-
-int checkbegin(const char* response, const char* pattern) {
-	if (strlen(response) < strlen(pattern)) {
-		return 0;
-	}
-	return !strncasecmp(response, pattern, strlen(pattern));
-}
-
-
 /* returns the code in a response */
 
 int getcode(const char* response) {
@@ -425,11 +435,11 @@ int parsesock(char* buffer, struct sockaddr_in *sin, int mode) {
 			return 0;
 		}
 		else {
-			log(3, "Invalid address in the PASV or PORT command: %s",
+			jlog(3, "Invalid address in the PASV or PORT command: %s",
 					buffer);
 		}
 	}
-	log(3, "Error parsing IP and port from %s", buffer);
+	jlog(3, "Error parsing IP and port from %s", buffer);
 	return -1;
 }
 
@@ -545,7 +555,7 @@ int cmp_domains(const char* name, const char* pattern) {
 }
 
 void err_time_readline(int fd) {
-	log(2, "Timeout reached in readline()");
+	jlog(2, "Timeout reached in readline()");
 	say(fd, "500 Connection timed out\r\n");
 }
 
@@ -559,7 +569,7 @@ void err_readline(int fd) {
 	if (!(err = get_errstr())) {
 		err = strerror(e);
 	}
-	log(2, s, err);
+	jlog(2, s, err);
 	msize = strlen(s) + strlen(err) + 7;
 	m = (char*) malloc(msize);
 	enough_mem(m);
@@ -669,7 +679,7 @@ unsigned int extract_port(const char* s) {
 	if (!t) {
 		return 0;
 	}
-	i = sscanf(t, "%d", &pno);
+	i = sscanf(t, "%u", &pno);
 	if ( i != 1 ) {
 		return 0;
 	}
@@ -721,11 +731,7 @@ char* extract_file(const char* pathfile) {
 int cryptcmp(const char* encrypted, const char* clear) {
 	const char* cmp;
 #ifdef HAVE_CRYPT
-	char salt[3];
-	salt[0] = encrypted[0];
-	salt[1] = encrypted[1];
-	salt[2] = '\0';
-	cmp = crypt(clear, salt);
+	cmp = crypt(clear, encrypted);
 #else
 	cmp = clear;
 #endif
@@ -785,17 +791,19 @@ void encrypt_password() {
 }
 
 
-char* to_ascii(char *data, int *len) {
+char* to_ascii(char *data, int *len, int strictconversion) {
 	int count, len2 = *len;
 	char* buffer2 = (char*) malloc(*len * 2);
 	int last = 0;
 	char *b2ptr = buffer2;
 
 	for (count = 0; count < len2; count++) {
-		if ((data[count] == 10) && (last != 13)) {
-			*b2ptr = 13;
-			b2ptr++;
-			(*len)++;
+		if ((data[count] == 10)) {
+			if (strictconversion || last != 13) {
+				*b2ptr = 13;
+				b2ptr++;
+				(*len)++;
+			}
 		}
 		*b2ptr = data[count];
 		last = *b2ptr;
@@ -805,32 +813,16 @@ char* to_ascii(char *data, int *len) {
 }
 
 
-char* from_ascii(char *data, int *len) {
-	char *dptr = data;
-	int count, len2 = *len;
-
-	for (count = 0; count < len2; count++) {
-		if (data[count] != 13) {
-			*dptr = data[count];
-			dptr++;
-		} else {
-			(*len)--;
-		}
-	}
-	return data;
-}
-
-
 FILE* open_logfile(const char* fname) {
 	FILE* logf;
 	int err;
 
-	umask(0x077);
+	umask(0077);
 	logf = fopen(fname, "a");
-	umask(0x000);
+	umask(0000);
 	err = errno;
 	if (!logf) {
-		log(1, "Couldn't open the log file %s: %s", fname,
+		jlog(1, "Couldn't open the log file %s: %s", fname,
 				strerror(errno));
 	}
 	errno = err;
@@ -924,7 +916,7 @@ int change_root(const char* stage) {
 	}
 	if (geteuid() != 0) {
 		if (getuid() != 0) {
-			log(4, "Not root - no attempt to chroot()");
+			jlog(4, "Not root - no attempt to chroot()");
 			return 0;
 		}
 		/* getuid == 0 but geteuid != 0 */
@@ -938,16 +930,16 @@ int change_root(const char* stage) {
 
 	i = chdir(directory);
 	if (i) {
-		log(4, "Could not chdir to %s", directory);
+		jlog(4, "Could not chdir to %s", directory);
 		changeid(UNPRIV, EUID, "After chroot (failed)");
 		return -1;
 	}
 	i = chroot(directory);
 	if (!i) {
-		log(7, "Changed root directory to %s", directory);
+		jlog(7, "Changed root directory to %s", directory);
 		srvinfo.chrooted = 1;
 	} else {
-		log(4, "Error changeing root directory to %s: %s",
+		jlog(4, "Error changeing root directory to %s: %s",
 						directory, strerror(errno));
 		changeid(UNPRIV, EUID, "After chroot (failed)");
 		return -1;
@@ -957,7 +949,7 @@ int change_root(const char* stage) {
 	}
 	i = chdir("/");
 	if (i) {
-		log(4, "Could not chdir to / of the chrooted environment");
+		jlog(4, "Could not chdir to / of the chrooted environment");
 		return -1;
 	}
 	return 0;
@@ -980,9 +972,9 @@ int dropprivileges(const char* stage) {
 			if (getuid() == 0) {
 				/* don't bring up a message if we aren't
 				 * root anyway */
-				log(2, "Option runasuser not set but dropping "
+				jlog(2, "Option runasuser not set but dropping "
 					"privileges requested - terminating");
-				log(2, "If you don't have a dropprivileges option in your configuration file at all, this is because dropprivileges is a standard option. If you really want to run as root, set \"dropprivileges never\" in the configuration file.");
+				jlog(2, "If you don't have a dropprivileges option in your configuration file at all, this is because dropprivileges is a standard option. If you really want to run as root, set \"dropprivileges never\" in the configuration file.");
 				return -1;
 			}
 		}
@@ -1028,8 +1020,10 @@ unsigned long int get_uint_ip(int type, struct clientinfo* clntinfo) {
 
 const char* get_char_ip(int type, struct clientinfo *clntinfo) {
 	unsigned long int ip = get_uint_ip(type, clntinfo);
+	struct in_addr addr;
+	addr.s_addr = ip;
 
-	return inet_ntoa(*((struct in_addr*) &ip));
+	return inet_ntoa(addr);
 }
 
 /*
@@ -1038,10 +1032,11 @@ const char* get_char_ip(int type, struct clientinfo *clntinfo) {
  */
 const char *get_char_peer_ip(int fd) {
 	unsigned long int ip;
+	struct in_addr addr;
 
 	ip = get_uint_peer_ip(fd);
-
-	return inet_ntoa(*((struct in_addr*) &ip));
+	addr.s_addr = ip;
+	return inet_ntoa(addr);
 }
 
 
@@ -1059,7 +1054,7 @@ unsigned long int get_uint_peer_ip(int fd) {
 	namelen = sizeof(name);
 
 	if (getpeername(fd, (struct sockaddr *) &name, &namelen) != 0) {
-		log(2, "Could not get peername: %s", strerror(errno));
+		jlog(2, "Could not get peername: %s", strerror(errno));
 		return UINT_MAX;
 	}
 
@@ -1101,7 +1096,7 @@ int get_interface_name(const struct sockaddr_in sin_req, char* iface) {
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 
 	if (fd < 0) {
-		log(7, "Failed to create socket in get_interface_name");
+		jlog(7, "Failed to create socket in get_interface_name");
 		return -1;
 	}
 
@@ -1117,7 +1112,7 @@ int get_interface_name(const struct sockaddr_in sin_req, char* iface) {
 		enough_mem(ifc.ifc_buf);
 
 		if ((ret = ioctl(fd, SIOCGIFCONF, &ifc)) < 0) {
-			log(5, "ioctl error: %s", strerror(errno));
+			jlog(5, "ioctl error: %s", strerror(errno));
 			break;
 		}
 		if (ifc.ifc_len == sizeof(struct ifreq) * nr) {
@@ -1141,7 +1136,7 @@ int get_interface_name(const struct sockaddr_in sin_req, char* iface) {
 			if (sin.sin_addr.s_addr == sin_req.sin_addr.s_addr) {
 				snprintf(iface, IF_NAMESIZE, 
 							"%s", ifr->ifr_name);
-				log(8, "Interface %s was IP %s", ifr->ifr_name,
+				jlog(8, "Interface %s was IP %s", ifr->ifr_name,
 					inet_ntoa(sin.sin_addr));
 				found = 1;
 				break;
@@ -1162,7 +1157,9 @@ int get_interface_name(const struct sockaddr_in sin_req, char* iface) {
 
 
 const char* conv_ip_to_char(unsigned long int ip) {
-	return inet_ntoa(*((struct in_addr*) &ip));
+	struct in_addr addr;
+	addr.s_addr = ip;
+	return inet_ntoa(addr);
 }
 
 
@@ -1224,3 +1221,15 @@ char* strnulldup(const char* str) {
 	return s;
 }
 
+char* strfilldup(const char* str, const char* fill) {
+	char* s;
+	if ( ! str ) {
+		s = strdup(fill);
+	} else {
+		s = strdup(str);
+	}
+	enough_mem(s);
+
+	return s;
+}
+
